<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Command;

use App\Entity\CollegueReminderDTO;
use App\Entity\InviteConsultantToken;
use App\Repository\InviteTokenRepository;
use App\Repository\ReminderLogRepository;
use App\Repository\UserReminderRepository;
use App\Entity\BeneficiaireAdminReminderDTO;
use Psr\Log\LoggerInterface;
use Symfony\Component\Console\Attribute\AsCommand;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\DependencyInjection\ParameterBag\ContainerBagInterface;
use Symfony\Component\Mailer\Exception\TransportExceptionInterface;
use Symfony\Component\Mailer\MailerInterface;
use Symfony\Component\Mime\Address;
use Symfony\Component\Mime\Email;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;

#[AsCommand(
    name: 'app:remind-beneficiaire',
    description: 'send email reminder to beneficiaire user',
)]
class RemindBeneficiaireCommand extends Command
{
    private string $diagUrl;

    public function __construct(
        private MailerInterface        $mailer,
        private LoggerInterface        $reminderLogger,
        private UrlGeneratorInterface  $urlGenerator,
        private ContainerBagInterface  $params,
        private UserReminderRepository $userReminderRepository,
        private InviteTokenRepository $inviteTokenRepository,
        private ReminderLogRepository $reminderLogRepository,
    )
    {
        parent::__construct();

        $this->diagUrl = $this->urlGenerator->generate('communeEPCI_diagnostics',[], UrlGeneratorInterface::ABSOLUTE_URL);
    }

    protected function configure(): void
    {
        $this->addArgument('beneficiaire', InputArgument::REQUIRED, 'Send reminder to admin or collegue');
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $io = new SymfonyStyle($input, $output);
        $beneficiaire = $input->getArgument('beneficiaire'); // admin|collegue
        $textFormat = $text = $subject = '';

        if ($beneficiaire === 'admin') {
            $rows = $this->userReminderRepository->findBeneficiaireAdminToSendReminder();
            $textFormat = $this->emailTextFormatForAdmin();
            $subject = 'Finalisez votre questionnaire CycloDiag';
        } elseif ($beneficiaire === 'collegue') {
            $rows = $this->userReminderRepository->findCollegueToSendReminder();
            $textFormat = $this->emailTextFormatForCollegue();
            $subject = 'Apportez vos réponses au questionnaire CycloDiag';
        }

        if (empty($rows)) {
            $io->success('no user to remind.');

            return Command::SUCCESS;
        }

        foreach ($rows as $userReminder) { // BeneficiaireAdminReminderDTO $userReminder
//            dd($userReminder);
            if ($beneficiaire === 'admin') {
                $text = $textFormat;
            } elseif ($beneficiaire === 'collegue') {
                $connectLik = $this->createLink($userReminder);
                $text = sprintf($textFormat, $connectLik);
            }
            $email = (new Email())
                ->from(new Address('mailer@cyclodiag.lafabriquedelalogistique.fr', 'CycloDiag Mailer'))
                ->to(new Address($userReminder->email, $userReminder->fullName))
                ->subject($subject)
                ->text($text);
//            dump($email);
            try {
                $this->mailer->send($email);
                $this->reminderLogger->log('info', 'reminder sent to {email}', [
                    'email' => $userReminder->email
                ]);
                $this->logReminder($userReminder);
            } catch (TransportExceptionInterface $e) {
                throw $e; // exception in command is logged by Symfony
            }
        }
//die;
        return Command::SUCCESS;
    }

    private function logReminder(BeneficiaireAdminReminderDTO|CollegueReminderDTO $userReminder)
    {
        $userReminder->setSent();
        $this->reminderLogRepository->save($userReminder);
    }

    private function emailTextFormatForAdmin()
    {

        return <<<TXT
Bonjour,

Vous avez démarré le questionnaire CycloDiag de la Fabrique de la Logistique et il vous reste quelques rubriques à compléter.
Nous vous invitons à finaliser votre évaluation via ce lien vers votre page : {$this->diagUrl}

Si vous avez besoin d'assistance ou avez des questions, écrivez-nous à : contact@fablog.io

Cordialement,
La Communauté Cyclologistique de la Fabrique de la Logistique 
TXT;

    }

    private function emailTextFormatForCollegue(): string
    {
        return <<<TXT
Bonjour,

Vous avez avez été sollicité(e) pour contribuer au questionnaire CycloDiag de la Fabrique de la Logistique.
Nous vous invitons à finaliser vos réponses via ce lien :  %s

Cordialement,
La Communauté Cyclologistique de la Fabrique de la Logistique 
TXT;
    }

    private function createLink(CollegueReminderDTO $userReminder): string
    {
        $inviteToken = InviteConsultantToken::create($userReminder->userId, $userReminder->diagUuid, $userReminder->qsToRespond[0]);

        $inviteToken->setVerifier($this->params->get('kernel.secret'));

        $this->inviteTokenRepository->removeInviteToken($inviteToken);
        $this->inviteTokenRepository->save($inviteToken);

        return $this->urlGenerator->generate('consultant_connect_for_response', [
            'publicToken' => $inviteToken->getPublicToken()
        ], UrlGeneratorInterface::ABSOLUTE_URL);
    }
}
