<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Controller;

use App\Entity\FicheAction;
use App\Form\FicheActionType;
use App\Menu\MenuBuilder;
use App\Repository\FicheActionRepository;
use App\Validator\UniqueReference;
use Assert\LazyAssertionException;
use Symfony\Component\Form\FormFactoryInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Symfony\Component\Security\Csrf\CsrfToken;
use Symfony\Component\Security\Csrf\CsrfTokenManagerInterface;
use Symfony\Component\Validator\Validator\ValidatorInterface;
use Twig\Environment;

#[AsController]
#[Route('/admin/fiche-action/new', name: 'admin_fiche_action_create', methods: ['GET', 'POST'])]
final class AdminFicheActionCreateController
{
    public function __construct(
        private Environment               $twig,
        private FormFactoryInterface      $formFactory,
        private ValidatorInterface        $validator,
        private CsrfTokenManagerInterface $csrfTokenManager,
        private UrlGeneratorInterface     $urlGenerator,
        private MenuBuilder               $menuBuilder,
        private FicheActionRepository     $ficheActionRepository,
    )
    {
    }

    public function __invoke(Request $request): Response
    {
        $menu = $this->menuBuilder->buildMenu('Fiches Action');
        $menu['Fiches Action']['subMenu']['Créer']['active'] = true;

        $default = [];
        if($request->getMethod() === 'GET') {
            $maxViewOrder = $this->ficheActionRepository->getMaxViewOrder();
            $default = [
              'viewOrder' => $maxViewOrder + 1
            ];
        }

        $form = $this->formFactory->createNamed('', FicheActionType::class, $default);

        $form->handleRequest($request);

        if ($request->getMethod() === 'POST') {

            $submittedToken = $request->getPayload()->get('_token');
            if (!$this->csrfTokenManager->isTokenValid(new CsrfToken('fiche_action_token', $submittedToken))) {
                $request->getSession()->getFlashBag()->add('error', 'Le jeton CSRF est invalide. Essayez de re-soumettre le formulaire');
            }

            try {
                $ficheAction = FicheAction::fromRequest($request);
                // validating that ficheAction reference is unique
                // doing it here bc could’nt instantiate the validator in FicheAction DTO (we need the ficheActionRepo)
                // anyway this make sense since we only want to validate uniqueness at creation
                $errors = $this->validator->validate($ficheAction->reference, new UniqueReference());
                if (count($errors) > 0) {
                    $request->getSession()->getFlashBag()->add('error', (string)$errors);
                }
            } catch (LazyAssertionException|\OutOfBoundsException $e) {

                $request->getSession()->getFlashBag()->add('error', $e->getMessage());
            }

            if ($request->getSession()->getFlashBag()->has('error')) {

                return new Response(
                    $this->twig->render('admin_fiche_action/create.html.twig', [
                        'menu' => $menu,
                        'ficheActionForm' => $form->createView(),
                    ]),
                    Response::HTTP_UNPROCESSABLE_ENTITY
                );
            } else {

                $this->ficheActionRepository->save($ficheAction);

                return new RedirectResponse(
                    $this->urlGenerator->generate('admin_fiche_action_list'),
                    Response::HTTP_CREATED
                );
            }
        }

        return new Response(
            $this->twig->render('admin_fiche_action/create.html.twig', [
                'menu' => $menu,
                'ficheActionForm' => $form->createView(),
            ]),
        );
    }
}
