<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Controller;

use App\Menu\MenuBuilder;
use App\Questionnaire\QuestionnaireCycloDiag;
use App\Questionnaire\QuestionnaireStep;
use App\Repository\CommuneEPCIRepository;
use App\Repository\DiagnosticCycloDiagRepository;
use App\Repository\InviteTokenRepository;
use App\Repository\ResponseCycloDiagRepository;
use Symfony\Bundle\SecurityBundle\Security;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Symfony\Component\Security\Http\Attribute\IsGranted;
use Twig\Environment;
use Twig\Extra\Intl\IntlExtension;

#[AsController]
#[IsGranted('ROLE_BENEFICIAIRE')]
#[Route('/commune-epci/diagnostics', name: 'communeEPCI_diagnostics', methods: ['GET'])]
final class CommuneEPCIDiagnosticListController
{
    private array $questionnaireSubMenu;

    private array $stepCompletion;

    public function __construct(
        private Environment                   $twig,
        private Security                      $security,
        private UrlGeneratorInterface         $urlGenerator,
        private DiagnosticCycloDiagRepository $diagnosticICLRepository,
        private ResponseCycloDiagRepository   $responseICLRepository,
        private CommuneEPCIRepository $communeEPCIRepository,
        private InviteTokenRepository         $inviteTokenRepository,
        private QuestionnaireCycloDiag        $questionnaireICL,
        private MenuBuilder                   $menuBuilder,
    )
    {
    }

    public function __invoke(): Response
    {
        // list all diags
        // for each diag find completion step
        // for each diag find invited user / question
        $this->twig->addExtension(new IntlExtension());
        $navMenu = $this->menuBuilder->buildMenu('Compte');
        $navMenu['Compte']['subMenu']['Diagnostics']['active'] = true;

        $user = $this->security->getUser();
        $collectiviteId = $user->getCommuneEPCIId();
        $allDiags = $this->diagnosticICLRepository->findAll($collectiviteId);
        foreach (array_keys($allDiags) as $diagUuid) {

            // will set stepCompletion AND navMenu['Questionnaire']['subMenu']
            $this->buildCompletionForDiag($diagUuid);

            $allDiags[$diagUuid]->addCompletions($this->stepCompletion);

            $navMenu['Questionnaire']['subMenu'] = $this->questionnaireSubMenu;

            $consultantsQuestions = $this->inviteTokenRepository->findAllConsultantsAndQuestions($diagUuid);
            $allDiags[$diagUuid]->addConsultantsQuestions($consultantsQuestions);
        }

        return new Response($this->twig->render('communeEPCI/diagnostics.html.twig', [
            'allDiags' => $allDiags,
            'menu' => $navMenu,
            'collectiviteMeta' => $this->communeEPCIRepository->getMeta($collectiviteId),

        ]));
    }

    private function buildCompletionForDiag($diagUuid)
    {
        $this->stepCompletion = $this->questionnaireSubMenu = [];
        $completions = $this->responseICLRepository->getCompletionForDiag($diagUuid);

        foreach (QuestionnaireStep::array() as $stepName => $label) {
            $completion = (isset($completions[$stepName])) ? (int)$completions[$stepName] : 0;
            $completion = $this->computeCompletionPercent($stepName, $completion);
            $this->stepCompletion[$stepName] = $completion;

            $menuLink = [
                'path' => $this->urlGenerator->generate('communeEPCI_questionnaire', [
                    'stepName' => $stepName,
                    'diagUuid' => $diagUuid,
                ]),
                'label' => $label,
                'completion' => $completion,
            ];
            $this->questionnaireSubMenu[] = $menuLink;
        }
    }

    private function computeCompletionPercent(string $stepName, int $completion): string
    {
        $enumStep = constant('App\Questionnaire\QuestionnaireStep::' . $stepName);
        $stepQuestionsCount = iterator_count($this->questionnaireICL->getQuestionsForStep($enumStep));

        if ((int)ceil($completion / $stepQuestionsCount * 100) === 100 ){
            return 100;
        }
        return "$completion/$stepQuestionsCount" ;

//        return (int)ceil($completion / $stepQuestionsCount * 100);
    }


}