<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Controller;

use App\Entity\DiagnosticCycloDiag;
use App\Entity\ResponseCycloDiag_DTO;
use App\Questionnaire\DiagnosticName;
use App\Questionnaire\QuestionInputName;
use App\Questionnaire\QuestionnaireCycloDiag;
use App\Questionnaire\QuestionnaireStep;
use App\Repository\CommuneEPCIRepository;
use App\Repository\DiagnosticCycloDiagRepository;
use App\Repository\ResponseCycloDiagRepository;
use Symfony\Bundle\SecurityBundle\Security;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Symfony\Component\Uid\Uuid;
use Twig\Environment;

#[AsController]
#[Route('/commune-epci/duplicate-diag', name: 'communeEPCI_duplicate_diag', methods: ['POST'])]
final class CommuneEPCIDuplicateDiagController
{


    public function __construct(
        private Security                      $security,
        private UrlGeneratorInterface         $urlGenerator,
        private Environment                   $twig,
        private DiagnosticCycloDiagRepository $diagRepository,
        private CommuneEPCIRepository         $communeEPCIRepository,
        private ResponseCycloDiagRepository   $responseCycloDiagRepository,
    )
    {
    }

    public function __invoke(Request $request): RedirectResponse|Response
    {
        $diagUuid = $request->request->get('diagUuid');

        if ($diagUuid === null) {

            return new RedirectResponse(
                $this->urlGenerator->generate('communeEPCI_diagnostics')
            );
        }

        $diagComplete = $this->isDiagComplete($diagUuid);

        if ($request->request->has('confirmDuplicate')
        || $diagComplete === true) {
            $user = $this->security->getUser();

            $diag = $this->diagRepository->findOne($diagUuid);
            $collectivite = $this->communeEPCIRepository->findOne($diag->communeEPCI_id);

            $newDiag = DiagnosticCycloDiag::create([
                'uuid' => Uuid::v4()->toRfc4122(),
                'communeEPCI_id' => $diag->communeEPCI_id,
                'ownerId' => $user->getId(),
                'name' => (string)DiagnosticName::create($collectivite),
            ]);
            $newDiag->setDuplicated($diagUuid);
            $this->diagRepository->save($newDiag);

            $response1 = ResponseCycloDiag_DTO::create(
                $newDiag->uuid,
                QuestionInputName::create('contexte-i-nom_commune'),
                QuestionnaireStep::CONTEXTE
            );
            $response1->setValue($collectivite->getName());
            $this->responseCycloDiagRepository->save($response1);

            $response2 = ResponseCycloDiag_DTO::create(
                $newDiag->uuid,
                QuestionInputName::create('contexte-i-region'),
                QuestionnaireStep::CONTEXTE
            );
            $response2->setValue($collectivite->getRegion());
            $this->responseCycloDiagRepository->save($response2);

            $request->getSession()->getFlashBag()->add('success', $newDiag->name . 'créé. Vos pouvez commencer à le remplir.');

            return new RedirectResponse(
                $this->urlGenerator->generate('communeEPCI_diagnostics')
            );
        }

        //        check that diag is complete
        if ( !$diagComplete ) {
            $warningMessage = <<<MSG
            Le diagnostic n’est pas complet. Certaines des questions n’ont pas été répondues.
            Confirmez si vous souhaitez recommencer le diagnostic tout de même.
            MSG;
            if ($request->headers->has('HX-Request')) {

                return new Response($this->twig->render('communeEPCI/_modal_confirm_duplicate.html.twig', [
                    'warningMessage' => $warningMessage,
                    'diagUuid' => $diagUuid,
                ]));
            } else {
                $request->getSession()->getFlashBag()->add('warning', $warningMessage);

                return new RedirectResponse(
                    $this->urlGenerator->generate('communeEPCI_diagnostics')
                );
            }
        }
    }

    private function isDiagComplete(string $diagUuid): bool
    {
        $questionsCount = (int)(new QuestionnaireCycloDiag())->count();
        $respondedQuestionsCount = (int)$this->responseCycloDiagRepository->getCountForDiag($diagUuid);

        return $questionsCount == $respondedQuestionsCount;
    }
}
