<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Controller;

use App\Entity\InviteConsultantRequestDTO;
use App\Entity\InviteConsultantToken;
use App\Entity\User;
use App\Entity\UserBeneficiaireRole;
use App\Repository\DiagnosticCycloDiagRepository;
use App\Repository\InviteTokenRepository;
use App\Repository\UserRepository;
use Symfony\Component\DependencyInjection\ParameterBag\ContainerBagInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Twig\Environment;

#[AsController]
#[Route('/commune-epci/make-invite', name: 'communeEPCI_invite_consultant', methods: ['POST'])]
final class CommuneEPCIInviteConsultantController
{
    public function __construct(
        private Environment                   $twig,
        private ContainerBagInterface         $params,
        private UrlGeneratorInterface         $urlGenerator,
        private DiagnosticCycloDiagRepository $diagnosticICLRepository,
        private InviteTokenRepository         $inviteTokenRepository,
        private UserRepository                $userRepository,
        private UserPasswordHasherInterface   $userPasswordHasher,
    ){}

    public function __invoke(Request $request): Response
    {
        /*
            dd($request->request->all());
         * array:3 [
              "consultantNameEmail" => "Martin Dupont <martin.dupont@ville-nantes.fr>"
              "diagUuid" => "5b0d7a08-512b-4aa9-9760-a86d16910052"
              "questionInputName" => "contexte-i-population"
            ]
         * */
        /* @TODO
         * UI dialog form name <email>
         *     on submit responds with email content & invite link
         *     + button "copy"
         *     + script for copy
         *
         * Invite link redirect to communeEPCI_questionnaire/step/diag?question=<questionInputName>
         * page communeEPCI_questionnaire/step/diag?question=<questionInputName>
         *     class .no-response on form != <questionInputName>
         *     .no-response {opacity: .8} .no-response:hover {opacity: 1}
         *     form == <questionInputName>
         *         focused / scrolled to
         *         Fiche Action open
         *  voter on ResponseToQuestion
         *
         *      Create Invite record
         *          Invite Token see <>
         *              https://paragonie.com/blog/2017/02/split-tokens-token-based-authentication-protocols-w            ithout-side-channels
         *           bin2hex(random_bytes(32))
         * */
        $errorMessage = [];

        try {
            $inviteConsultantRequest = InviteConsultantRequestDTO::fromRequest($request);
        } catch (\Exception|\OutOfBoundsException $e) {
            $errorMessage[] = $e->getMessage();
        }

        if (count($errorMessage) > 0) {
            $errorMessage = implode('<br>', $errorMessage);

            return new Response(<<<HTML
                <div class="alert form-error">
                    $errorMessage
                </div>
HTML,
                Response::HTTP_BAD_REQUEST);
        }

        $communeEPCIId = $this->diagnosticICLRepository->findCommuneIdByUuid($inviteConsultantRequest->getDiagUuid());

        $user = $this->isUserEmailUnique($inviteConsultantRequest->getConsultantEmail());

        // no unique email requirement, we can request a consultant multiple time
        if ($user === null) {
            $user = new User();
            $user->setCommuneEPCIid($communeEPCIId);
            $user->setRoles([UserBeneficiaireRole::ROLE_BENEFICIAIRE->value]);
            $user->setFullname($inviteConsultantRequest->getConsultantName());
            $user->setEmail($inviteConsultantRequest->getConsultantEmail());
            $user->setPassword(
                $this->userPasswordHasher->hashPassword(
                    $user,
                    bin2hex(random_bytes(8))
                ));
            $this->userRepository->save($user, true);
        }

        $inviteToken = InviteConsultantToken::create($user->getId(), $inviteConsultantRequest->getDiagUuid(), $inviteConsultantRequest->getQuestionInputName());

        $inviteToken->setVerifier($this->params->get('kernel.secret'));
        /* dd($inviteToken);
          -verifier: "5998c036d1188f88127c8742"
          +selector: "16bee687"
          +userId: 34
          +diagUuid: "6648a5fa-eb0f-4629-8283-6e5b6c4611ea"
          +questionInputName: "contexte-i-population"
          +expiresAt: "2024-06-03 13:58:54"
          -hashedVerifier: "21a26b66265fa366fda4ca676a719712b2304eb727c1ec568c6a3edd11f61b96"
         * */

        $this->inviteTokenRepository->removeInviteToken($inviteToken);

        $this->inviteTokenRepository->save($inviteToken);

        return new Response($this->twig->render('communeEPCI/_modal_invite_mail_content.html.twig', [
            'responseUrl' => $this->urlGenerator->generate('consultant_connect_for_response', [
                'publicToken' => $inviteToken->getPublicToken()
            ], UrlGeneratorInterface::ABSOLUTE_URL)
        ]));
    }

    private function isUserEmailUnique(string $emailToCheck): ?User
    {
        $mayBeUser = $this->userRepository->findOneBy([
            'email' => $emailToCheck,
        ]);

        return $mayBeUser;
    }
}
