<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Controller;

use App\Config\navMenu;
use App\Entity\ResponseCycloDiag_DTO;
use App\Entity\UserBeneficiaireRole;
use App\Menu\MenuBuilder;
use App\Questionnaire\QuestionnaireCycloDiag;
use App\Questionnaire\QuestionnaireStep;
use App\Repository\DiagnosticCycloDiagRepository;
use App\Repository\FicheActionRepository;
use App\Repository\InviteTokenRepository;
use App\Repository\ResponseCycloDiagRepository;
use App\Security\DiagnosticVoter;
use Symfony\Bundle\SecurityBundle\Security;
use Symfony\Component\Form\FormFactoryInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;
use Symfony\Component\Security\Http\Attribute\IsGranted;
use Twig\Environment;

#[AsController]
#[IsGranted('ROLE_BENEFICIAIRE')]
#[Route('/commune-epci/questionnaire/{stepName}/{diagUuid}', name: 'communeEPCI_questionnaire', methods: ['GET'])]
final class CommuneEPCIQuestionnaireController
{
    private string $questionnaire_uuid;
    private QuestionnaireCycloDiag $questionnaireCycloDiag;
    private QuestionnaireStep $currentStep;
    private $stepCompletion = 'En cours';
    private $ficheActionRefs = [];


    public function __construct(
        private readonly ResponseCycloDiagRepository   $responseICLRepository,
        private readonly DiagnosticCycloDiagRepository $diagnosticICLRepository,
        private readonly FicheActionRepository         $ficheActionRepository,
        private readonly InviteTokenRepository         $inviteTokenRepository,
        private MenuBuilder                            $menuBuilder,
        private FormFactoryInterface                   $formFactory,
        private Security                               $security,
        private Environment                            $twig,
        private UrlGeneratorInterface                  $urlGenerator,
    )
    {
        $this->questionnaireCycloDiag = new QuestionnaireCycloDiag();
    }

    public function __invoke(string $stepName, string $diagUuid): Response
    {
        $diagnostic = $this->diagnosticICLRepository->findOne($diagUuid);

        // @See ADR in wiki
        $this->denyAccessUnlessGranted(DiagnosticVoter::RESPOND, $diagnostic);
        $userId = $this->security->getUser()->getId();
        $questionRespondedByConsultant = $this->inviteTokenRepository->findQuestionsFor($userId, $diagUuid);
        //dd($questionRespondedByConsultant); // array of (string)QuestionInputName

        $this->currentStep = constant('App\Questionnaire\QuestionnaireStep::' . $stepName);
        $questions = [];
        $responses = $this->responseICLRepository->getValuesVersionForDiag($diagUuid); // $inputName => $value

        $formNbr = 0;
        foreach ($this->questionnaireCycloDiag->getQuestionsForStep($this->currentStep) as $question) {
            $questionVersion = 0;
            $formNbr++;
            $questionInputName = (string)$question->getInputName();

            // we collect all the ficheActionRefs for preview display
            $faRef = $question->resolveFicheAction();
            if ($faRef !== null &&
                !in_array($faRef, $this->ficheActionRefs)) {
                $this->ficheActionRefs[] = $faRef;
            }

            $responseDTO = ResponseCycloDiag_DTO::create($diagUuid, $question->getInputName(), $this->currentStep);
            if (isset($responses[$questionInputName])) {
                $responseDTO->setValue($responses[$questionInputName]['value']);
                $responseDTO->setOther($responses[$questionInputName]['other']);
                $questionVersion = $responses[$questionInputName]['version'];
            }

            if ($questionRespondedByConsultant
                && in_array($questionInputName, $questionRespondedByConsultant)) {
                $question->addAttr(['class' => 'response-waiting']);
            }
            // named form to help w/ debugging (in symfony debugbar)
            // @WARNING tight coupling with css for conditional field
            // if any part of the name change, check for form:has(#question-2_diagnostic-reglementaire-circulation_1:checked) in app.css
            $form = $this->formFactory->createNamed('question-' . $formNbr,
                $question->formClass, $responseDTO, [
                    'action' => $this->urlGenerator->generate('communeEPCI_questionnaire_question'),
                    'method' => 'POST',
                    'question_name' => (string)$question->getInputName(),
                    'label' => $question->label,
                    'label_help' => $question->help,
                    'other_option' => $question->otherOption,
                    'attr' => $question->getAttr(),
                    'ficheActionRef' => $question->resolveFicheAction(),
                ]);
            $questions[] = [
                'form' => $form->createView(),
                'sectionHead' => $question->sectionHead,
                'stepName' => $question->getStep()->name,
                'questionInputName' => $questionInputName,
                'questionVersion' => $questionVersion,
            ];
        }

        return new Response(
            $this->twig->render('communeEPCI/questionnaire.html.twig', [
                'questions' => $questions,
                'prevNext' => QuestionnaireStep::prevNext($this->currentStep),
                'diagUuid' => $diagUuid,
                'menu' => $this->buildMenu($diagUuid),
                'stepCompletion' => $this->stepCompletion,
                'ficheActionPreview' => $this->ficheActionPreview(),
            ]));
    }

    private function denyAccessUnlessGranted($attribute, $subject)
    {
        if (!$this->security->isGranted($attribute, $subject)) {

            $exception = new AccessDeniedException('Vous n’avez pas les droits pour remplir ce questionnaire');
            $exception->setAttributes([$attribute]);
            $exception->setSubject($subject);

            throw $exception;
        }
    }

    private function buildMenu(string $diagUuid): array
    {
        $navMenu = $this->menuBuilder->buildMenu('Questionnaire');
        $menuLink = [
            'path' => null,
            'label' => '',
            'active' => false,
            'completion' => 0
        ];
        $subMenu = [];

        $completions = $this->responseICLRepository->getCompletionForDiag($diagUuid);

        foreach (QuestionnaireStep::array() as $stepName => $label) {
            $completion = (isset($completions[$stepName])) ? (int)$completions[$stepName] : 0;
            $completion = $this->computeCompletionPercent($stepName, $completion);
            if ((int)$completion === 100) {
                $this->stepCompletion = 'Terminé';
            }

            $menuLink = [
                'path' => $this->urlGenerator->generate('communeEPCI_questionnaire', [
                    'stepName' => $stepName,
                    'diagUuid' => $diagUuid,
                ]),
                'label' => $label,
                'active' => $this->currentStep->name === $stepName,
                'completion' => $completion,
                'stepName' => $stepName,
            ];
            $subMenu[] = $menuLink;
        }
        $navMenu['Questionnaire']['subMenu'] = $subMenu;

        return $navMenu;
    }

    private function computeCompletionPercent(string $stepName, int $completion): string
    {
        $enumStep = constant('App\Questionnaire\QuestionnaireStep::' . $stepName);
        $stepQuestionsCount = iterator_count($this->questionnaireCycloDiag->getQuestionsForStep($enumStep));

        if ((int)ceil($completion / $stepQuestionsCount * 100) === 100 ){
            return 100;
        }

                return "$completion/$stepQuestionsCount" ;
    }

    private function ficheActionPreview()
    {
        return $this->ficheActionRepository->findFicheActionPreviewByRefs(
            $this->ficheActionRefs);
    }
}
