<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Controller;

use App\Entity\IndiceOperateur;
use App\Menu\MenuBuilder;
use App\Repository\CommuneEPCIRepository;
use App\Repository\IndiceOperateurRepository;
use Assert\LazyAssertionException;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Symfony\Component\Security\Csrf\CsrfToken;
use Symfony\Component\Security\Csrf\CsrfTokenManagerInterface;
use Twig\Environment;

#[Route('/indice-operateur/{publicId}', name: 'indice_operateur', methods: ['GET', 'POST'])]
#[AsController]
final class IndiceOperateurController
{
    public function __construct(
        private Environment               $twig,
        private CsrfTokenManagerInterface $csrfTokenManager,
        private UrlGeneratorInterface     $urlGenerator,
        private MenuBuilder               $menuBuilder,
        private CommuneEPCIRepository     $communeEPCIRepository,
        private IndiceOperateurRepository $indiceOperateurRepository,
    )
    {
    }

    public function __invoke(Request $request, $publicId = null): Response|RedirectResponse
    {
        $menu = $this->menuBuilder->buildMenu('Indice Opérateur');
        if ($request->getMethod() === 'POST') {
            /* values from form
                'collectivite' => 'Toulouse',
                'entreprise' => 'coursier toulouse',
                'reglementation_rank' => '2',
                'reglementation_commentaire' => 'Il sort par la pluie, il marche dans l’eau, il voyage en hiver. Il n’a pas peur de la nuit, des routes suspectes ni des rencontres',
                'foncier_rank' => '3',
                'foncier_commentaire' => 'Voilà comme il faut être avec un homme qui a du grand dans l’esprit.',
                'ammenagement_securite_rank' => '4',
                'ammenagement_lieux_rank' => '3',
                'ammenagement_stationnement_rank' => '5',
                'ammenagement_commentaire' => 'Il y avait près de Digne, dans la campagne, un homme qui vivait solitaire',
                'politique_rank' => '3',
                'politique_commentaire' => 'Cet homme, disons tout de suite le gros mot, était un ancien conventionnel. Il se nommait G.',
                'emploi_rank' => '2',
                'emploi_commentaire' => 'Était-ce du reste un vautour que G.? Oui, si l’on en jugeait par ce qu’il y avait de farouche dans sa solitude',
                'investissement_rank' => '3',
                'investissement_commentaire' => 'L’évêque prit son bâton, mit son pardessus à cause de sa soutane un peu trop usée',
                'communication_rank' => '5',
                'communication_commentaire' => 'C’était une cabane toute basse, indigente, petite et propre, avec une treille clouée à la façade.',
                'priorite_levier' => [
                    0 => 'Réglementation',
                    1 => 'Foncier',
                    2 => 'Aménagement',
                    3 => 'Emploi',
                    4 => 'Investissement',
                    5 => 'Communication'],
                '_collectivite_public_id' => nullOr 'lkmk',
                '_indice_ope_token' => 'bfad6a1aa3709d07a3ea113f1b7304aa.P8tp8FQnuKyS6HFajwpf_qQbALlLKhL38teyvV6oIVI.B7xfoQ1L2e3TrkIW-G4xyvR0ctg7WirHl7LAym2ZaTNmmziWA13Zx6SfIA'
            ]; * */

            $submittedToken = $request->request->get('_indice_ope_token');
            if (!$this->csrfTokenManager->isTokenValid(new CsrfToken('indice-ope', $submittedToken))) {
                $request->getSession()->getFlashBag()->add('error', 'Le jeton CSRF est invalide. Essayez de re-soumettre le formulaire');
            }

            try {
                $indiceOperateur = IndiceOperateur::fromRequest($request);
            } catch (LazyAssertionException $e) {
                $request->getSession()->getFlashBag()->add('error', $this->formatError($e));

                return new Response(
                    $this->twig->render('indice_operateur.html.twig', [
                        'menu' => $menu
                    ]));
            }

            if ($request->request->has('_collectivite_public_id')) {
                // get collectivite_id
                $publicId = $request->request->get('_collectivite_public_id');
                $collectiviteId = $this->communeEPCIRepository->getIdByPublicId($publicId);
                $indiceOperateur->setCollectiviteId($collectiviteId);
            }

            $this->indiceOperateurRepository->save($indiceOperateur);

            $request->getSession()->getFlashBag()->add('success', 'Merci d’avoir participé ! Votre réponse est bien enregistrée.');

            return new RedirectResponse(
                $this->urlGenerator->generate('home')
            );
        }

        return new Response(
            $this->twig->render('indice_operateur.html.twig', [
                'menu' => $menu,
                'publicId' => $publicId
            ]));
    }

    private function formatError(LazyAssertionException|\Exception $e)
    {
        $message = array_map(function ($error) {
            return $error->getMessage();
        }, $e->getErrorExceptions());

        return implode("\n", $message);
    }
}
