<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Controller;

use App\Entity\ResponseCycloDiag_DTO;
use App\Questionnaire\Question;
use App\Questionnaire\QuestionInputName;
use App\Questionnaire\QuestionnaireCycloDiag;
use App\Questionnaire\QuestionnaireStep;
use App\Repository\DiagnosticCycloDiagRepository;
use App\Repository\ResponseCycloDiagRepository;
use App\Repository\ReminderLogRepository;
use Symfony\Bundle\SecurityBundle\Security;
use Symfony\Component\Form\FormFactoryInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Symfony\Component\Security\Http\Attribute\IsGranted;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Validator\Validator\ValidatorInterface;
use Twig\Environment;

#[AsController]
#[IsGranted('ROLE_BENEFICIAIRE')]
#[Route('/commune-epci/questionnaire-question/{inputName}', name: 'communeEPCI_questionnaire_question', methods: ['POST'])]
final class QuestionnaireQuestionController
{
    const PRECISER = 'Preciser'; // used by NoYesSpecifyType|YesNoOtherType AND 'otherOption'
    public function __construct(
        private Environment                   $twig,
        private FormFactoryInterface          $formFactory,
        private UrlGeneratorInterface         $urlGenerator,
        private ValidatorInterface            $validator,
        private Security $security,
        private DiagnosticCycloDiagRepository $diagnosticICLRepository,
        private ResponseCycloDiagRepository   $responseICLRepository,
        private ReminderLogRepository   $reminderLogRepository,
    )
    {
    }

    public function __invoke(Request $request, string $inputName = ''): Response
    {
        $responseICL = ResponseCycloDiag_DTO::fromRequest($request);

        $errorMessage = [];
        $responseStatus = null;
        $notBlank = new Assert\NotBlank([
            'message' => 'Ce champ ne peut pas être vide'
        ]);
        $errors = $this->validator->validate(
            $responseICL->getValue(),
            $notBlank
        );

        //  when processing request > case for "preciser" input
        $questionInputName = QuestionInputName::create($responseICL->inputName);
        $question = (new QuestionnaireCycloDiag())->getQuestion($questionInputName);
        $formClass = $question->formClass;

        if ((
                $formClass === 'App\Questionnaire\Form\NoYesSpecifyType'
                && $responseICL->getValue() === 'oui'
            ) || (
                $formClass === 'App\Questionnaire\Form\YesNoOtherType'
                && $question->otherOption === 'Autre (veuillez préciser)'
                && $responseICL->getValue() === 'Autre (veuillez préciser)')
        ) {
            $formName = $request->request->keys()[0];
            $other = $request->request->all($formName)[self::PRECISER];
            $notBlank->message = 'Veuillez remplir le champ “préciser”';
            $specifyValidation = $this->validator->validate(
                $other,
                $notBlank,
            );
            if (!$specifyValidation->count()){
                $responseICL->setOther($other);
            } else {
               $errors[] = $specifyValidation->get(0);
            }
        }

        if (!$errors->count()) {
            // update diag updatedAt
            $this->diagnosticICLRepository->updatedAt($responseICL->diagUuid);
            // store response
            try {
                $this->responseICLRepository->save($responseICL);
                $responseStatus = 'Réponse enregistrée';
                $this->removeReminderLogRecord();
            } catch (\Exception $e) {
                // add to errorMessage
                $errorMessage[] = $e->getMessage();
            }
        } else {
            $errorMessage[] = $errors[0]->getMessage();
        }

        $questionInputName = QuestionInputName::create($responseICL->inputName);
        $question = (new QuestionnaireCycloDiag())->getQuestion($questionInputName);


        // form submitted via HTMX Ajax request
        if ($request->headers->has('HX-Request')) {
            $formName = $request->request->keys()[0];
            $form = $this->formFactory->createNamed($formName,
                $question->formClass,
                $responseICL, [
                    'action' => $this->urlGenerator->generate('communeEPCI_questionnaire_question'),
                    'method' => 'POST',
                    'question_name' => (string)$question->getInputName(),
                    'label' => $question->label,
                    'label_help' => $question->help,
                    'other_option' => $question->otherOption,
                    'attr' => $question->getAttr()
                ]);
            // @TODO rerun completion

            return new Response(
                $this->twig->load('communeEPCI/_questionnaire_question.html.twig')->renderBlock('body', [
                    'questionForm' => $form->createView(),
                    'questionInputName' => $questionInputName,
                    'sectionHead' => $question->sectionHead,
                    'questionVersion' => $responseICL->version(),
                    'ficheActionRef' => $question->resolveFicheAction(),
                    'diagUuid' => $responseICL->diagUuid,
                    'stepName' => $question->getStep()->name,
                    'errorMessage' => $errorMessage, // array
                    'responseStatus' => $responseStatus,
                    'subLink' => $this->buildSubLink($responseICL, $question),
                ]));
        } else {
            if (count($errorMessage) > 0) {
                // we'll use this in case of redirect flash notif will be rendered in the form template
                $formName = $request->request->keys()[0];
                $request->getSession()->getFlashBag()->add('errorMessage', [
                    'formName' => $formName,
                    'message' => $errorMessage, // array
                ]);
            }
            return new RedirectResponse(
                $this->urlGenerator->generate('communeEPCI_questionnaire', [
                    'stepName' => $question->getStep()->name,
                    'diagUuid' => $responseICL->diagUuid,
                ])
            );
        }
    }

    private function buildSubLink(ResponseCycloDiag_DTO $responseICL, Question $question): array
    {
        $completions = $this->responseICLRepository->getCompletionForDiag($responseICL->diagUuid);
        $stepName = $question->getStep()->name;
            $completion = (isset($completions[$stepName])) ? (int)$completions[$stepName] : 0;
            $completion = $this->computeCompletionPercent($question->getStep(), $completion);

            return [
                'path' => $this->urlGenerator->generate('communeEPCI_questionnaire', [
                    'stepName' => $stepName,
                    'diagUuid' => $responseICL->diagUuid,
                ]),
                'label' => $question->getStep()->value,
                'active' => true,
                'completion' => $completion,
                'stepName' => $stepName,
            ];
    }
    private function computeCompletionPercent(QuestionnaireStep $step, int $completion): string
    {
        $stepQuestionsCount = iterator_count((new QuestionnaireCycloDiag())->getQuestionsForStep($step));

        if ((int)ceil($completion / $stepQuestionsCount * 100) === 100 ){
            return 100;
        }

        return "$completion/$stepQuestionsCount" ;
    }

    private function removeReminderLogRecord()
    {
        $user = $this->security->getUser();
        $this->reminderLogRepository->delete($user->getId());
    }

}
