<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Controller;

use App\Entity\CommuneEPCI;
use App\Entity\DiagnosticCycloDiag;
use App\Entity\ResponseCycloDiag_DTO;
use App\Entity\User;
use App\Entity\UserBeneficiaireRole;
use App\Form\RegistrationFormType;
use App\Questionnaire\DiagnosticName;
use App\Questionnaire\QuestionInputName;
use App\Questionnaire\QuestionnaireStep;
use App\Repository\CommuneEPCIRepository;
use App\Repository\DiagnosticCycloDiagRepository;
use App\Repository\ResponseCycloDiagRepository;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Bundle\SecurityBundle\Security;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Uid\Uuid;

class RegistrationController extends AbstractController
{

    #[Route('/creer-compte', name: 'app_register')]
    public function register(
        Request                       $request,
        UserPasswordHasherInterface   $userPasswordHasher,
        EntityManagerInterface        $entityManager,
        Security                      $security,
        CommuneEPCIRepository         $communeEPCIRepository,
        DiagnosticCycloDiagRepository $diagnosticICLRepository,
        ResponseCycloDiagRepository   $responseICLRepository,
    ): Response
    {
        $user = new User();
        $form = $this->createForm(RegistrationFormType::class);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            if ($form->get('bestTimeContact')->getData() === true) { // honeypot filled by robot

                return $this->redirectToRoute('home');
            }
            // check unique user email
            if (!$this->isUserEmailUnique($form, $entityManager)) {
                $request->getSession()->getFlashBag()->add('warning', 'Cet email est déjà utilisé.');

                return $this->render('registration/register.html.twig', [
                    'registrationForm' => $form,
                ]);
            }
            // set commune_EPCI
            $communeEPCI = CommuneEPCI::create(
                $form->get('nomCollectivite')->getData(),
                $form->get('region')->getData()
            );
            $communeEPCIRepository->save($communeEPCI);
            $communeEPCIId = $communeEPCIRepository->lastInsertedId();

            $user->setCommuneEPCIid($communeEPCIId);
            $user->setRoles([UserBeneficiaireRole::ROLE_BENEFICIAIRE_ADMIN->value]);
            $user->setFullname($form->get('fullname')->getData());
            $user->setEmail($form->get('email')->getData());
            $user->setPassword(
                $userPasswordHasher->hashPassword(
                    $user,
                    $form->get('plainPassword')->getData()
                )
            );
            $entityManager->persist($user);
            $entityManager->flush();

            //create diag
            $diag = DiagnosticCycloDiag::create([
                'uuid' => Uuid::v4()->toRfc4122(),
                'communeEPCI_id' => $communeEPCIId,
                'ownerId' => $user->getId(),
                'name' => (string) DiagnosticName::create($communeEPCI),
                ]);
            $diagnosticICLRepository->save($diag);

            // fullfill 2 fist question
            $response1 = ResponseCycloDiag_DTO::create(
                $diag->uuid,
                QuestionInputName::create('contexte-i-nom_commune'),
                QuestionnaireStep::CONTEXTE
            );
            $response1->setValue($communeEPCI->getName());
            $responseICLRepository->save($response1);

            $response2 = ResponseCycloDiag_DTO::create(
                $diag->uuid,
                QuestionInputName::create('contexte-i-region'),
                QuestionnaireStep::CONTEXTE
            );
            $response2->setValue($communeEPCI->getRegion());
            $responseICLRepository->save($response2);

            $security->login($user, 'form_login');
            $request->getSession()->getFlashBag()->add('success', 'Votre compte est créé ! Vous pouvez commencer à utiliser le questionnaire.');

            return $this->redirectToRoute('communeEPCI_questionnaire', [
                'stepName' => QuestionnaireStep::CONTEXTE->name,
                'diagUuid' => $diag->uuid,
            ]);
        }

        return $this->render('registration/register.html.twig', [
            'registrationForm' => $form,
        ]);
    }

    private function isUserEmailUnique(\Symfony\Component\Form\FormInterface $form, EntityManagerInterface $entityManager): bool
    {
        $emailToCheck = $form->get('email')->getData();
        $mayBeUser = $entityManager->getRepository(User::class)->findOneBy([
            'email' => $emailToCheck,
        ]);
        return $mayBeUser === null;
    }

}
