<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Controller;

use App\Menu\MenuBuilder;
use App\Repository\CommuneEPCIRepository;
use App\Repository\IndiceOperateurRepository;
use Symfony\Bundle\SecurityBundle\Security;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\Routing\Attribute\Route;
use Twig\Environment;

#[AsController]
#[Route('/results/indice-operateur', name: 'results_indice_operateurs')]
final class ResultsIndiceOperateurController
{
    public function __construct(
        private Environment               $twig,
        private Security                  $security,
        private MenuBuilder               $menuBuilder,
        private IndiceOperateurRepository $indiceOperateurRepository,
        private CommuneEPCIRepository     $communeEPCIRepository,
    )
    {
    }

    public function __invoke(): Response
    {
        $isBeneficiaire = $this->security->isGranted('ROLE_BENEFICIAIRE_ADMIN');

        $topLevel = ($isBeneficiaire) ? 'Compte' : 'Tableau de bord';
        $menu = $this->menuBuilder->buildMenu($topLevel);
        $menu[$topLevel]['subMenu']['Indice Opérateurs']['active'] = true;

        $resultsForCollectivite = $allResultsSorted = [];
        $collectivitePublicId = null;

        if ($isBeneficiaire) {
            $collectiviteId = $this->security->getUser()->getCommuneEPCIId();
            $collectivitePublicId = $this->getOrMakePublicId($collectiviteId);

            $resultsForCollectivite = $this->indiceOperateurRepository->findAllByCollectivite($collectiviteId);
        }
        $allResultsSorted = $this->indiceOperateurRepository->findAllSorted();

        return new Response(
            $this->twig->render('results/indice_operateurs.html.twig', [
                'menu' => $menu,
                'resultsForCollectivite' => $resultsForCollectivite,
                'allResultsSorted' => $allResultsSorted,
                'collectivitePublicId' => $collectivitePublicId,
            ])
        );
    }

    private function getOrMakePublicId(int $collectiviteId): string
    {
        $collectivitePublicId = $this->communeEPCIRepository->getPublicIdById($collectiviteId);

        if ($collectivitePublicId === null) {
            $collectivitePublicId = bin2hex(random_bytes(8));
            $this->communeEPCIRepository->updatePublicId($collectiviteId, $collectivitePublicId);
        }

        return $collectivitePublicId;
    }
}
