<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Entity;

use App\Validator\UniqueReference;
use Assert\Assert;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\String\Slugger\AsciiSlugger;
use Symfony\Component\HtmlSanitizer\HtmlSanitizerConfig;
use Symfony\Component\HtmlSanitizer\HtmlSanitizer;

//use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Validator\Validation;

class FicheAction
{
    /**
     * @var null
     */
    private static $config = null;
    private static $sanitizer = null;
    private function __construct(
        public readonly string $reference, // notBlank, uniqueReference
        private string         $theme, // valid choice
        private string         $preview, // notBlank
        private string         $title, // title, content, theme must either all blank or all notEmpty
        private string         $content, //
        private string         $viewOrder, // nullOrInt
        private string         $slug, //
        private int            $version, // int
    )
    {
    }

    public static function fromRequest(Request $request): self
    {
        $reference = $request->request->get('reference') ?? throw new \OutOfBoundsException('Il manque la reference');
        $viewOrder = (int)$request->request->get('viewOrder') ?? throw new \OutOfBoundsException('Il manque l’ordre');
        $title = $request->request->get('title') ?? throw new \OutOfBoundsException('Il manque le titre');
        $theme = $request->request->get('theme') ?? throw new \OutOfBoundsException('Il manque la thématique');
        $content = $request->request->get('content') ?? throw new \OutOfBoundsException('Il manque le contenu');
        $preview = $request->request->get('preview') ?? throw new \OutOfBoundsException('Il manque la synthèse');
        $version = (int)$request->request->get('version') ?? 0;

        self::validate($reference, $viewOrder, $title, $theme, $content, $preview, $version);

        $slug = self::slugify($title);
        $preview = self::sanitizeHTML($preview);
        $content = self::sanitizeHTML($content);

        return new self(
            $reference,
            $theme,
            $preview,
            $title,
            $content,
            $viewOrder,
            $slug,
            $version,
        );
    }

    private static function validate($reference, $viewOrder, $title, $theme, $content, $preview, $version)
    {
        Assert::lazy()->tryAll()
            ->that($reference, 'reference')->notBlank()
            ->that($preview, 'preview')->notBlank()
            ->that($viewOrder, 'viewOrder')->nullOr()->integer()->min(0)
            ->that($version, 'version')->notBlank()->integer()->min(0)
            ->verifyNow(); // will throw in the controller when trying to create

        if ($theme != '' || $title != '' || $content != '') {
            Assert::lazy()->tryAll()
                ->that($theme, 'theme')->notBlank()->choice(['Aide à l’investissement', 'Aménagement du territoire', 'Foncier', 'Portage politique'])
                ->that($title, 'title')->notBlank()->minLength(5)
                ->that($content, 'content')->notBlank()->maxLength(20000)
                ->verifyNow(); // will throw in the controller when trying to create
        }

    }


    public static function fromDB($row): self
    {
        return new self(
            $row['reference'],
            $row['theme'],
            $row['preview'],
            $row['title'],
            $row['content'],
            $row['viewOrder'],
            $row['slug'],
            $row['version'],
        );
    }

    public function toDBRecord(): array
    {
        $this->version = ($this->version === null) ? 0 : ++$this->version;

        return [
            'reference' => $this->reference,
            'theme' => $this->theme,
            'preview' => $this->preview,
            'title' => $this->title,
            'content' => $this->content,
            'viewOrder' => $this->viewOrder,
            'slug' => $this->slug,
            'version' => $this->version,
        ];
    }

    public function toFormData()
    {
        return [
            'reference' => $this->reference,
            'theme' => $this->theme,
            'preview' => $this->preview,
            'title' => $this->title,
            'content' => $this->content,
            'viewOrder' => $this->viewOrder,
            'slug' => $this->slug,
            'version' => $this->version,
        ];
    }

    public function identifier(): array
    {
        return [
            'reference' => $this->reference,
        ];
    }

    public static function tableName(): string
    {
        return 'ficheAction';
    }

    public static function params()
    {
        return [
            \PDO::PARAM_STR,
            \PDO::PARAM_STR,
            \PDO::PARAM_STR,
            \PDO::PARAM_STR,
            \PDO::PARAM_STR,
            \PDO::PARAM_STR,
            \PDO::PARAM_STR,
            \PDO::PARAM_INT,
        ];

    }

    private static function slugify(float|bool|int|string $title)
    {
        $slugger = new AsciiSlugger();
        return $slugger->slug($title);
    }

    private static function sanitizeHTML(string $raw)
    {
        if (self::$config === null) {
            self::$config = (new HtmlSanitizerConfig())
                ->allowSafeElements()
                ->allowStaticElements();
        }
        if (self::$sanitizer === null) {
            self::$sanitizer = new HtmlSanitizer(self::$config);
        }

        return self::$sanitizer->sanitize($raw);
    }

    public function version()
    {
        return $this->version;
    }


}
