<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Entity;

use Symfony\Component\HtmlSanitizer\HtmlSanitizer;
use Symfony\Component\HtmlSanitizer\HtmlSanitizerConfig;
use Symfony\Component\HttpFoundation\Request;
use Assert\Assert;


class IndiceOperateur
{
    private static $config = null;
    private static $sanitizer = null;

    private function __construct(
        private string $collectivite,
        private string $entreprise,
        private int    $reglementation_rank,
        private string $reglementation_commentaire,
        private int    $foncier_rank,
        private string $foncier_commentaire,
        private int    $ammenagement_securite_rank,
        private int    $ammenagement_lieux_rank,
        private int    $ammenagement_stationnement_rank,
        private string $ammenagement_commentaire,
        private int    $politique_rank,
        private string $politique_commentaire,
        private int    $emploi_rank,
        private string $emploi_commentaire,
        private int    $investissement_rank,
        private string $investissement_commentaire,
        private int    $communication_rank,
        private string $communication_commentaire,
        private string $priorite_levier_1,
        private string $priorite_levier_2,
        private string $priorite_levier_3,
        private string $priorite_levier_4,
        private string $priorite_levier_5,
        private string $priorite_levier_6,
        private ?int   $collectiviteId = null,
    )
    {
    }

    public static function fromRequest(Request $request): self
    {

        $collectivite = $request->request->get('collectivite') ?? '';
        $entreprise = $request->request->get('entreprise') ?? '';
        $reglementation_rank = (int)$request->request->get('reglementation_rank') ?? 0;
        $reglementation_commentaire = $request->request->get('reglementation_commentaire') ?? '';
        $foncier_rank = (int)$request->request->get('foncier_rank') ?? 0;
        $foncier_commentaire = $request->request->get('foncier_commentaire') ?? '';
        $ammenagement_securite_rank = (int)$request->request->get('ammenagement_securite_rank') ?? 0;
        $ammenagement_lieux_rank = (int)$request->request->get('ammenagement_lieux_rank') ?? 0;
        $ammenagement_stationnement_rank = (int)$request->request->get('ammenagement_stationnement_rank') ?? 0;
        $ammenagement_commentaire = $request->request->get('ammenagement_commentaire') ?? '';
        $politique_rank = (int)$request->request->get('politique_rank') ?? 0;
        $politique_commentaire = $request->request->get('politique_commentaire') ?? '';
        $emploi_rank = (int)$request->request->get('emploi_rank') ?? 0;
        $emploi_commentaire = $request->request->get('emploi_commentaire') ?? '';
        $investissement_rank = (int)$request->request->get('investissement_rank') ?? 0;
        $investissement_commentaire = $request->request->get('investissement_commentaire') ?? '';
        $communication_rank = (int)$request->request->get('communication_rank') ?? '';
        $communication_commentaire = $request->request->get('communication_commentaire') ?? '';
        $priorite_levier = $request->request->all('priorite_levier') ?? [];

        Assert::lazy()->tryAll()
            ->that($collectivite, 'collectivite')->notBlank()
            ->that($entreprise, 'entreprise')->notBlank()
            ->that($reglementation_rank, 'reglementation_rank')->integer()->between(1, 5, self::rangeErrorMessage('reglementation'))
            ->that($foncier_rank, 'foncier_rank')->integer()->between(1, 5, self::rangeErrorMessage('foncier'))
            ->that($ammenagement_securite_rank, 'ammenagement_securite_rank')->integer()->between(1, 5, self::rangeErrorMessage('ammenagement securite'))
            ->that($ammenagement_lieux_rank, 'ammenagement_lieux_rank')->integer()->between(1, 5, self::rangeErrorMessage('ammenagement lieux'))
            ->that($ammenagement_stationnement_rank, 'ammenagement_stationnement_rank')->integer()->between(1, 5, self::rangeErrorMessage('ammenagement stationnement'))
            ->that($politique_rank, 'politique_rank')->integer()->between(1, 5, self::rangeErrorMessage('politique'))
            ->that($emploi_rank, 'emploi_rank')->integer()->between(1, 5, self::rangeErrorMessage('emploi'))
            ->that($investissement_rank, 'investissement_rank')->integer()->between(1, 5, self::rangeErrorMessage('investissement'))
            ->that($communication_rank, 'communication_rank')->integer()->between(1, 5, self::rangeErrorMessage('communication'))
            ->that($priorite_levier, 'priorite_levier')->isArray()->satisfy(function ($priorite_levier) {
                $priorities = ['Foncier', 'Aménagement', 'Réglementation', 'Emploi', 'Investissement', 'Communication'];
                $verify = true;
                foreach ($priorite_levier as $priorite) {
                    if (!in_array($priorite, $priorities)) {
                        $verify = false;
                        break;
                    }
                }
                return $verify;
            }, function () {
                return 'Une des valeurs de levier d’action ne correspond pas à la liste';
            })
            ->verifyNow(); // will throw in the controller when trying to create

        return new self(
            $collectivite,
            $entreprise,
            $reglementation_rank,
            $reglementation_commentaire,
            $foncier_rank,
            $foncier_commentaire,
            $ammenagement_securite_rank,
            $ammenagement_lieux_rank,
            $ammenagement_stationnement_rank,
            $ammenagement_commentaire,
            $politique_rank,
            $politique_commentaire,
            $emploi_rank,
            $emploi_commentaire,
            $investissement_rank,
            $investissement_commentaire,
            $communication_rank,
            $communication_commentaire,
            $priorite_levier[0],
            $priorite_levier[1],
            $priorite_levier[2],
            $priorite_levier[3],
            $priorite_levier[4],
            $priorite_levier[5],
        );

    }

    public function setCollectiviteId(int $collectiviteId): void
    {
        $this->collectiviteId = $collectiviteId;
    }

    public function toDBRecord(): array
    {
        return [
            'collectivite' => $this->collectivite,
            'entreprise' => $this->entreprise,
            'reglementation_rank' => $this->reglementation_rank,
            'reglementation_commentaire' => $this->sanitizeHTML($this->reglementation_commentaire),
            'foncier_rank' => $this->foncier_rank,
            'foncier_commentaire' => $this->sanitizeHTML($this->foncier_commentaire),
            'ammenagement_securite_rank' => $this->ammenagement_securite_rank,
            'ammenagement_lieux_rank' => $this->ammenagement_lieux_rank,
            'ammenagement_stationnement_rank' => $this->ammenagement_stationnement_rank,
            'ammenagement_commentaire' => $this->sanitizeHTML($this->ammenagement_commentaire),
            'politique_rank' => $this->politique_rank,
            'politique_commentaire' => $this->sanitizeHTML($this->politique_commentaire),
            'emploi_rank' => $this->emploi_rank,
            'emploi_commentaire' => $this->sanitizeHTML($this->emploi_commentaire),
            'investissement_rank' => $this->investissement_rank,
            'investissement_commentaire' => $this->sanitizeHTML($this->investissement_commentaire),
            'communication_rank' => $this->communication_rank,
            'communication_commentaire' => $this->sanitizeHTML($this->communication_commentaire),
            'priorite_levier_1' => $this->priorite_levier_1,
            'priorite_levier_2' => $this->priorite_levier_2,
            'priorite_levier_3' => $this->priorite_levier_3,
            'priorite_levier_4' => $this->priorite_levier_4,
            'priorite_levier_5' => $this->priorite_levier_5,
            'priorite_levier_6' => $this->priorite_levier_6,
            'collectiviteId' => $this->collectiviteId,
        ];
    }


    private function sanitizeHTML(string $raw)
    {
        if (self::$config === null) {
            self::$config = (new HtmlSanitizerConfig())
                ->allowSafeElements();
        };
        if (self::$sanitizer === null) {
            self::$sanitizer = new HtmlSanitizer(self::$config);
        }

        return self::$sanitizer->sanitizeFor('textarea', $raw);
    }

    private static function rangeErrorMessage($propertyName)
    {

        return 'Le champ "' . $propertyName . '" doit comporter entre entre 1 et 5 étoiles';
    }

    public static function params(): array
    {
        return [
            \PDO::PARAM_STR,
            \PDO::PARAM_STR,
            \PDO::PARAM_INT,
            \PDO::PARAM_STR,
            \PDO::PARAM_INT,
            \PDO::PARAM_STR,
            \PDO::PARAM_INT,
            \PDO::PARAM_INT,
            \PDO::PARAM_INT,
            \PDO::PARAM_STR,
            \PDO::PARAM_INT,
            \PDO::PARAM_STR,
            \PDO::PARAM_INT,
            \PDO::PARAM_STR,
            \PDO::PARAM_INT,
            \PDO::PARAM_STR,
            \PDO::PARAM_INT,
            \PDO::PARAM_STR,
            \PDO::PARAM_STR,
            \PDO::PARAM_STR,
            \PDO::PARAM_STR,
            \PDO::PARAM_STR,
            \PDO::PARAM_STR,
            \PDO::PARAM_STR,
        ];
    }
}