<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Entity;


final class InviteConsultantToken
{
    private $verifier = null;
    private function __construct(
        public readonly string $selector,
        public readonly int    $userId,
        public readonly string $diagUuid,
        public readonly string $questionInputName,
        public readonly string $expiresAt,
        private  ?string $hashedVerifier = null,
    ){}

    public static function create(
                                  int    $userId,
                                  string $diagUuid,
                                  string $questionInputName,): self
    {
        $selector = bin2hex(random_bytes(4));
        $expiresAt = (new \DateTimeImmutable('+1 month'))->format('Y-m-d H:i:s');

        return new self(
            $selector,
            $userId,
            $diagUuid,
            $questionInputName,
            $expiresAt);
    }

    public function setVerifier($secret): void
    {
        $this->verifier = bin2hex(random_bytes(12));
        $this->hashedVerifier = hash_hmac('sha256', $this->verifier, $secret);
    }

    public function getPublicToken(): string
    {
        if ($this->verifier === null) {
            throw new \OutOfBoundsException('Call setVerifier before calling this method');
        }
        return $this->selector.$this->verifier;
    }

    public function verifyPublicToken(string $verifier, string $secret):bool
    {
        return  hash_equals(hash_hmac('sha256', $verifier, $secret), $this->hashedVerifier);
    }

    public function toDBRecord(): array
    {
        return [
            'selector' => $this->selector,
            'userId' => $this->userId,
            'diagUuid' => $this->diagUuid,
            'questionInputName' => $this->questionInputName,
            'expiresAt' => $this->expiresAt,
            'hashedVerifier' => $this->hashedVerifier,
        ];
    }

    public static function fromDB(array $row): self
    {
        return  new self(
            $row['selector'],
            $row['userId'],
            $row['diagUuid'],
            $row['questionInputName'],
            $row['expiresAt'],
            $row['hashedVerifier'],
        );
    }

    public function identifier()
    {
        return [
            'userId' => $this->userId,
            'diagUuid' => $this->diagUuid,
            'questionInputName' => $this->questionInputName,
        ];
    }

    public static function tableName(){
        return 'inviteConsultantToken';
    }



    public static function params()
    {
        return [
            \PDO::PARAM_STR,
            \PDO::PARAM_INT,
            \PDO::PARAM_STR,
            \PDO::PARAM_STR,
            \PDO::PARAM_STR,
            \PDO::PARAM_STR,
        ];
    }}