<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Entity;

use Assert\Assert;
use Symfony\Component\HtmlSanitizer\HtmlSanitizer;
use Symfony\Component\HtmlSanitizer\HtmlSanitizerConfig;
use Symfony\Component\HttpFoundation\Request;

class publicPage
{
    private function __construct(
        public readonly string $slug,
        public readonly string $html,
        private int    $version,
        public readonly ?string $createdAt,
    )
    {
    }

    public static function fromRequest(Request $request)
    {
        $slug = $request->request->get('slug') ?? throw new \OutOfBoundsException('Il manque slug');
        $html = $request->request->get('HTML') ?? throw new \OutOfBoundsException('Il manque HTML');
        $version = (int)$request->request->get('version') ?? throw new \OutOfBoundsException('Il manque version');

        Assert::lazy()->tryAll()
            ->that($slug, 'reference')->notBlank()
            ->that($html, 'preview')->notBlank()
            ->that($version, 'version')->notBlank()->integer()->min(0)
            ->verifyNow(); // will throw in the controller when trying to create

        return new self($slug, $html, $version, null);
    }

    public function toDBRecord(): array
    {
        $this->version = ($this->version === null) ? 0 : ++$this->version;

        return [
            'slug' => $this->slug,
            'html' => $this->sanitizeHTML($this->html),
            'version' => $this->version,
            'createdAt' => (new \DateTimeImmutable())->format('Y-m-d H:i'),
        ];
    }

    public static function fromDB($row): self
    {
        return new self(
            $row['slug'],
            $row['html'],
            $row['version'],
            $row['createdAt'],
        );
    }

    private function sanitizeHTML(string $raw)
    {
        $config = (new HtmlSanitizerConfig())
            ->allowSafeElements()
            ->allowStaticElements();
        $sanitizer = new HtmlSanitizer($config);
        return $sanitizer->sanitize($raw);
    }

    public function version(){
        return $this->version;
    }

    public function identifier(): array
    {
        return [
            'slug' => $this->slug,
            'version' => $this->version
        ];
    }

    public function isObsolete()
    {
        $now = new \DateTimeImmutable();
        $pageDate = new \DateTimeImmutable($this->createdAt);
        $i = $now->diff($pageDate);

        return (int)$i->format('%a') > 180;
    }

}