<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Form;

use App\Entity\User;
use App\Entity\UserBeneficiaireRole;
use App\Validator\IsNotFreeDomain;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\EmailType;
use Symfony\Component\Form\Extension\Core\Type\PasswordType;
use Symfony\Component\Form\Extension\Core\Type\RadioType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Validator\Constraints\Choice;
use Symfony\Component\Validator\Constraints\IsTrue;
use Symfony\Component\Validator\Constraints\Length;
use Symfony\Component\Validator\Constraints\NotBlank;

class RegistrationFormType extends AbstractType
{
    private $regions = [
                    'Auvergne-Rhône-Alpes' => 'Auvergne-Rhône-Alpes',
                    'Bourgogne-Franche-Comté' => 'Bourgogne-Franche-Comté',
                    'Bretagne' => 'Bretagne',
                    'Centre-Val de Loire' => 'Centre-Val de Loire',
                    'Corse' => 'Corse',
                    'Grand Est' => 'Grand Est',
                    'Hauts-de-France' => 'Hauts-de-France',
                    'Île-de-France' => 'Île-de-France',
                    'Normandie' => 'Normandie',
                    'Nouvelle-Aquitaine' => 'Nouvelle-Aquitaine',
                    'Occitanie' => 'Occitanie',
                    'Pays de la Loire' => 'Pays de la Loire',
                    'Provence-Alpes-Côte d’Azur' => 'Provence-Alpes-Côte d’Azur',
                    'Guadeloupe' => 'Guadeloupe',
                    'Martinique' => 'Martinique',
                    'Guyane' => 'Guyane',
                    'La Réunion' => 'La Réunion',
                    'Mayotte' => 'Mayotte',
                ];
    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $builder
            ->add('nomCollectivite', null, [
                'label' => 'Nom de la Commune ou de l’EPCI',
                'mapped' => false,
                'required' => true,
                'constraints' => [
                    new NotBlank([
                        'message' => 'Il manque le nom de la collectivité',
                    ]),
                ],
            ])
            ->add('region', ChoiceType::class, [
                'label' => 'Région',
                'mapped' => false,
                'choices' => array_merge(['choisir une région' => ''],$this->regions),
                'required' => true,
                'expanded' => false,
                'multiple' => false,
                'constraints' => [
                    new Choice([
                        'choices' => $this->regions,
                        'message' => 'Choisir une région dans la liste'
                    ]),
                ],
            ])
            ->add('fullname', null, [
                'label' => 'Nom complet',
                'required' => true,
                'constraints' => [
                    new NotBlank([
                        'message' => 'Merci de remplir votre nom complet',
                    ]),
                ],
            ])
            ->add('email', EmailType::class,[
                'attr' => ['autocomplete' => 'email'],
                'required' => true,
                'help' => 'Afin d’éviter les spam nous privilégions les adresse mail professionnelles',
                'constraints' => [
                    new NotBlank([
                        'message' => 'Merci de remplir votre email professionnel',
                    ]),
                    new IsNotFreeDomain()
                ],
            ])
            ->add('plainPassword', PasswordType::class, [
                // instead of being set onto the object directly,
                // this is read and encoded in the controller
                'mapped' => false,
                'required' => true,
                'label' => 'Mot de passe',
                'help' => 'Votre mot de passe doit avoir au moins 12 caractères',
                'attr' => [
                    'autocomplete' => 'new-password',
                    'minlength' => 12
                ],
                'constraints' => [
                    new NotBlank([
                        'message' => 'Merci de saisir un mot de passe',
                    ]),
                    new Length([
                        'min' => 12,
                        'minMessage' => 'Votre mot de passe doit avoir au moins {{ limit }} caractères',
                        // max length allowed by Symfony for security reasons
                        'max' => 4096,
                    ]),
                ],
            ])
            // honeypot
            ->add('bestTimeContact', RadioType::class, [
                'mapped' => false,
                'label' => 'Best time to contact',
                'required' => false,
                'label_attr' => [
                    'aria-hidden' => "true",
                    'tabindex' => '-1',
                    'style' => "display:none;",
                ],
                'attr' => [
                    'tabindex' => '-1',
                    'style' => "display:none;",
                ],
            ])
        ;
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
//            'data_class' => User::class,
            'csrf_protection' => true,
            'csrf_field_name' => '_token',
            'csrf_token_id'   => 'register_token',
        ]);
    }
}
