<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Questionnaire;

use Assert\Assert;
use Assert\AssertionFailedException;
use function Assert\that;

class PublicPageBuilder
{
    private static array $indexedResponses;
    private static array $notImplemented = [];

    public static function page(array $indexedResponses): array
    {
        self::$indexedResponses = $indexedResponses;
        try {
            Assert::lazy()->tryAll()
                ->that($indexedResponses, 'echelon')->keyIsset('contexte-i-echelon', function () {
                    return 'Il manque l’échelon';
                })
                ->that($indexedResponses, 'nom_commune')->keyIsset('contexte-i-nom_commune', function () {
                    return 'Il manque le nom de la commune';
                })
                ->that($indexedResponses, 'population')->keyIsset('contexte-i-population', function () {
                    return 'Il manque la population';
                })
                ->that($indexedResponses, 'superficie')->keyIsset('contexte-i-superficie', function () {
                    return 'Il manque la superficie';
                })
                ->that($indexedResponses, 'densite')->keyIsset('contexte-i-densite', function () {
                    return 'Il manque la densité';
                })
                ->verifyNow();
        } catch (AssertionFailedException $e) {
            $r = [];
            $r[] = 'Nous ne pouvons pas générer de page, il manque les informations suivantes :';
            foreach ($e->getErrorExceptions() as $errorException) {
                $r[] = $errorException->getMessage();
            }

            return [
                'page' => implode('<br>', $r),
                'notImplemented' => [],

            ];

        }

        $page = trim(self::identiteTerritoire());
        $page .= trim(self::restrictionCirculation());
        $page .= trim(self::cyclabilite());
        $page .= trim(self::entreprises());
        $page .= trim(self::portagePol());
        $page .= trim(self::polLogistique());
        $page .= trim(self::instancePartenariale());
        $page .= trim(self::ressourceCyclo());
        $page .= trim(self::suiviActiviteCyclo());
        $page .= trim(self::communication());
        $page .= trim(self::clauseCyclo());
        $page .= trim(self::aides());

        return [
            'page' => $page,
            'notImplemented' => self::$notImplemented
        ];
    }

    private static function identiteTerritoire()
    {
        $echelon = self::$indexedResponses['contexte-i-echelon']['value'];
        $nom = self::$indexedResponses['contexte-i-nom_commune']['value'];
        $population = self::$indexedResponses['contexte-i-population']['value'];
        $superficie = self::$indexedResponses['contexte-i-superficie']['value'];
        $densite = self::$indexedResponses['contexte-i-densite']['value'];

        return <<<HTML
<h3>Fiche d’identité cyclologistique du territoire</h3>
<p>La $echelon de $nom compte $population habitants sur une superficie de $superficie km² soit une densité de population de $densite.</p>
HTML;
    }

    private static function restrictionCirculation()
    {
        if (!isset(self::$indexedResponses['diagnostic-reglementaire-circulation'])) {
            return '';
        }
        $existe = (self::$indexedResponses['diagnostic-reglementaire-circulation']['value'] == 'non') ? false : true;
        if ($existe) {
            $c = self::$indexedResponses['diagnostic-reglementaire-circulation']['other'];
            $v = '';
            if (isset(self::$indexedResponses['diagnostic-reglementaire-circulation-motorisation'])) {
//                $v = self::$indexedResponses['diagnostic-reglementaire-circulation-motorisation']['value'];
                $v = ', qui interdisent aux vignettes Crit’Air <span class="to-review">(n°) à préciser</span> de circuler';
            }

            return <<<HTML
        <p>Il existe sur notre territoire des restrictions à la circulation des véhicules professionnels motorisés identifiées par <span class="to-review">$c</span>$v.</p>
HTML;
        } else {
            self::$notImplemented[] = 'restreindre la circulation des véhicules professionnels motorisés (ZTL, ZFE, centre piétonnier ..)';
            return '';
        }
    }

    private static function cyclabilite()
    {
        if (!isset(self::$indexedResponses['diagnostic-amenagement-indice_cycla-voirie'])) {
            return '';
        }

        $c = self::$indexedResponses['diagnostic-amenagement-indice_cycla-voirie']['value'];
        $t = '';
        if (isset(self::$indexedResponses['diagnostic-amenagement-indice_cycla-voirie_30'])) {
            $t = self::$indexedResponses['diagnostic-amenagement-indice_cycla-voirie_30']['value'];
            $t = ' et ' . $t . ' de la voirie est limitée à 30 km/h';
        }
        $b = '';
        if (isset(self::$indexedResponses['diagnostic-amenagement-indice_cycla-bidirec'])) {
            $b = self::$indexedResponses['diagnostic-amenagement-indice_cycla-bidirec']['value'];
            $b = ' et les aménagements cyclables bidirectionnels sont évalués à ' . $b . ' m/habitant.';
        }

        return <<<HTML
        <p>En ce qui concerne notre indice de cyclabilité, les voies cyclables représentent $c de la voirie$t$b.</p>
HTML;
    }

    private static function entreprises()
    {
        $n = 'Favoriser l’implantation d’entreprises réalisant des livraisons à vélo';
        if (!isset(self::$indexedResponses['diagnostic-i-nbre_ent'])) {
            self::$notImplemented[] = $n;
            return '';
        }
        if (self::$indexedResponses['diagnostic-i-nbre_ent'] == 0) {
            self::$notImplemented[] = $n;

            return '';
        }
        $c = self::$indexedResponses['diagnostic-i-nbre_ent']['value'];

        return <<<HTML
        <p>Nous accueillons sur le territoire $c entreprises réalisant des livraisons à vélo. </p>
HTML;
    }

    private static function portagePol()
    {
        $n = 'Porter le sujet de la cyclologistique en délibération';
        if (!isset(self::$indexedResponses['gouvernance-portage-deliberation'])) {
            self::$notImplemented[] = $n;

            return '';
        }
        $oui = (self::$indexedResponses['gouvernance-portage-deliberation']['value'] == 'oui') ? true : false;
        if ($oui) {

            return <<<HTML
<p>Sur les questions de portage politique, notre assemblée d’élus, impliquée sur les questions de cyclologistique a déjà délibéré sur ce sujet, c’était le <span class="to-review"> (date à préciser)</span> au sujet de <span class="to-review"> (détails à préciser)</span>. </p>
HTML;
        } else {
            self::$notImplemented[] = $n;

            return '';
        }
    }

    private static function polLogistique()
    {
        $n = 'Lancer une étude dédiée sur le sujet';

        if (!isset(self::$indexedResponses['gouvernance-portage-politique_cyclo'])) {

            self::$notImplemented[] = $n;
            return '';
        }
        $oui = (self::$indexedResponses['gouvernance-portage-politique_cyclo']['value'] == 'oui') ? true : false;
        if ($oui) {
            $c = self::$indexedResponses['gouvernance-portage-politique_cyclo']['other'];
            $a = (isset(self::$indexedResponses['gouvernance-autre-etude_cyclo'])) ? self::$indexedResponses['gouvernance-autre-etude_cyclo']['value'] : 'non';
            $a = ($a == 'oui') ? 'Une étude spécifique avait été menée sur ce sujet en <span class="to-review">Date à renseigner</span>' : '';

            return <<<HTML
            <p>Notre politique de logistique urbaine inclut la cyclologistique et prévoit <span class="to-review">$c</span>. $a </p>
HTML;
        } else {
            self::$notImplemented[] = $n;

            return '';
        }
    }

    private static function instancePartenariale()
    {
        $n = 'Participer aux instances partenariales comme le Programme InterLud+ ou la communauté cyclologistique de la Fabrique de la Logistique';

        if (!isset(self::$indexedResponses['concertation-sensibilisation-participation'])) {
            self::$notImplemented[] = $n;

            return '';
        }

        if (self::$indexedResponses['concertation-sensibilisation-participation']['value'] == 'oui') {

            $i = self::$indexedResponses['concertation-sensibilisation-participation']['other'];
            if (isset(self::$indexedResponses['concertation-sensibilisation-interlud'])
                && self::$indexedResponses['concertation-sensibilisation-interlud']['value'] === 'oui') {

                $i .= ', Interlud+';
            }
            if (isset(self::$indexedResponses['concertation-sensibilisation-fablog'])
                && self::$indexedResponses['concertation-sensibilisation-fablog']['value'] === 'oui') {
                $i .= ', communauté cyclologistique de la Fabrique de la logistique';
            }
            return <<<HTML
        <p>Nous participons aux instances partenariales suivantes sur le sujet de la cyclologistique : $i </p>
HTML;
        } else {
            self::$notImplemented[] = $n;

            return '';
        }
    }


    private static function ressourceCyclo()
    {
        $n = 'Nommer une ressource à la logistique urbaine avec une feuille de route qui intègre des actions spécifiques à la cyclologistique.';

        if (!isset(self::$indexedResponses['implementation-autre-rh-cyclo'])) {
            self::$notImplemented[] = $n;

            return '';
        }

        if (self::$indexedResponses['implementation-autre-rh-cyclo']['value'] == 'oui') {
            $f = '';
            if (self::$indexedResponses['implementation-autre-rh-action_cyclo'] != '') {
                $f = ' et sa/leur feuille de route intègre des actions spécifiques à la cyclologistique.';
            }
            return <<<HTML
<p>En termes d’investissement RH, nous disposons de <span class="to-review"> (nombre à préciser)</span> ressource(s) humaine(s) dédiée à la logistique urbaine$f.</p>
HTML;
        } else {
            self::$notImplemented[] = $n;

            return '';
        }
    }

    private static function suiviActiviteCyclo()
    {
        $n = 'mettre en place un dispositif permettant de suivre l’évolution des activités de cyclologistique sur le territoire';

        if (!isset(self::$indexedResponses['implementation-autre-suivi_cyclo'])) {
            self::$notImplemented[] = $n;

            return '';
        }
        if (self::$indexedResponses['implementation-autre-suivi_cyclo']['value'] == 'non') {
            self::$notImplemented[] = $n;

            return '';
        }
        $s = self::$indexedResponses['implementation-autre-suivi_cyclo']['other'];

        return <<<HTML
<p>Nous avons mis en place le dispositif suivant qui nous permet de suivre l’évolution des activités de cyclologistique sur le territoire : <span class="to-review"> $s</span>
</p>
HTML;
    }

    private static function communication()
    {
        $n = 'Communiquer sur les actions mises en place en faveur de la cyclologistique';

        if (!isset(self::$indexedResponses['implementation-sensibilisation-communication'])) {
            self::$notImplemented[] = $n;

            return '';
        }
        $oui = (self::$indexedResponses['implementation-sensibilisation-communication']['value'] == 'oui') ? true : false;
        $colisActiv = (self::$indexedResponses['implementation-autre-colis_activ']['value'] == 'oui') ? 'Colis Activ' : '';

        if ($oui) {
            return <<<HTML
<p>Pour ces raisons, la collectivité communique régulièrement sur les actions mises en place en faveur de la cyclologistique, notamment <span class="to-review"> $colisActiv (préciser)</span>.</p>
HTML;
        } else {
            self::$notImplemented[] = $n;

            return '';
        }
    }

    private static function clauseCyclo()
    {
        $n = 'Intégrer une clause spécifique à la cyclologistique dans nos marchés publics';

        if (!isset(self::$indexedResponses['implementation-autre-suivi_cyclo'])) {
            self::$notImplemented[] = $n;

            return '';
        }
        if (self::$indexedResponses['implementation-autre-suivi_cyclo']['value'] === 'oui') {

            $c = self::$indexedResponses['implementation-autre-suivi_cyclo']['other'];
            return <<<HTML
<p>Par ailleurs, nous avons intégré une clause spécifique à la cyclologistique dans nos marchés publics qui prévoit <span class="to-review">$c</span> </p>
HTML;
        } else {
            self::$notImplemented[] = $n;

            return '';
        }
    }

    private static function aides()
    {
        $n = 'soutenir l’installation et/ou l’investissement pour la cyclologistique.';
        $output = '';

        if (!isset(self::$indexedResponses['implementation-foncier-investissement-dispositifs'])) {
            self::$notImplemented[] = $n;

            $output = '';
        } else if (self::$indexedResponses['implementation-foncier-investissement-dispositifs']['value'] === 'oui') {

            $a = self::$indexedResponses['implementation-foncier-investissement-dispositifs']['other'];

            $output = <<<HTML
            <p>En ce qui concerne les aides, nous soutenons l’installation / l’investissement pour la cyclologistique au travers des dispositifs suivants : $a </p>
HTML;
        } else {
            self::$notImplemented[] = $n;

            $output = '';
        }

        $n = 'Proposer des locations longue durée de vélos-cargo pour les entreprises dans le cadre d’une activité de cyclologistique ainsi que des expérimentations courte durée de mise  à disposition de vélos-cargo.';

        $l = $e = 'non';
        if (isset(self::$indexedResponses['implementation-foncier-investissement-location'])) {
            $l = self::$indexedResponses['implementation-foncier-investissement-location']['value'];
        }
        if (isset(self::$indexedResponses['implementation-foncier-investissement-experimentation'])) {
            $e = self::$indexedResponses['implementation-foncier-investissement-experimentation']['value'];

        }


        if ($l === 'oui') {
            $output .= '<p>En outre, nous proposons pour les entreprises dans le cadre d’une activité de cyclologistique des locations longue durée de vélos-cargo';
        }
        if ($e === 'oui') {
            $output .= ' ainsi que des expérimentations courte durée de mise  à disposition de vélos-cargo';
        }
        $output .= '.</p>';
        if ($l === 'non' || $e === 'non') {
            self::$notImplemented[] = $n;
        }


        return $output;
    }
}
