<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Questionnaire;

final class QuestionnaireCycloDiag implements \IteratorAggregate
{

  private array $questions = [];
  private \ArrayObject $questionCollection;

  private $step;
  
  public function __construct()
  {

    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::CONTEXTE,
      'question_input_name' => 'contexte-i-echelon',
      'label' => '<b>1.1.</b> Échelon étudié',
      'formClass' => 'App\Questionnaire\Form\CommuneEPCIType',
      'ficheActionRef' => null,
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::CONTEXTE,
      'question_input_name' => 'contexte-i-nom_commune',
      'label' => '<b>1.2.</b> Nom de la Commune ou de l’EPCI',
      'formClass' => 'App\Questionnaire\Form\CycloDiagTextType',
      'ficheActionRef' => null,
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::CONTEXTE,
      'question_input_name' => 'contexte-i-region',
      'label' => '<b>1.3.</b> Region',
      'formClass' => 'App\Questionnaire\Form\CycloDiagTextType',
      'ficheActionRef' => null,
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::CONTEXTE,
      'question_input_name' => 'contexte-i-population',
      'label' => '<b>1.4.</b> Population (en nombre d’habitants)',
      'formClass' => 'App\Questionnaire\Form\CycloDiagNumberType',
      'ficheActionRef' => 'contexte-population',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::CONTEXTE,
      'question_input_name' => 'contexte-i-superficie',
      'label' => '<b>1.5.</b> Superficie (en km²)',
      'formClass' => 'App\Questionnaire\Form\CycloDiagNumberType',
      'ficheActionRef' => 'contexte-population',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::CONTEXTE,
      'question_input_name' => 'contexte-i-densite',
      'label' => '<b>1.6.</b> Densité de Population (habitants/km²)',
      'help' => '👉 Vous pouvez utiliser <a href="https://www.observatoire-des-territoires.gouv.fr/densite-de-population">l’Observatoire des territoires</a>',
      'formClass' => 'App\Questionnaire\Form\CycloDiagNumberType',
      'ficheActionRef' => 'contexte-population',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::DIAGNOSTIC,
      'question_input_name' => 'diagnostic-i-nbre_ent',
      'label' => '<b>2.1.</b> Nombre d’entreprises réalisant des livraisons à vélo implantées sur le territoire',
      'formClass' => 'App\Questionnaire\Form\CycloDiagNumberType',
      'ficheActionRef' => 'Realiser-diagnostic-cyclologistique',
    ]));
    $this->add(Question::fromArray([
        'step' => QuestionnaireStep::DIAGNOSTIC,
        // @WARNING tight coupling with css for conditional field
        // if any part of the name change, check for form:has(#question-2_diagnostic-reglementaire-circulation_0 in app.css
        'question_input_name' => 'diagnostic-reglementaire-circulation',
        'label' => '<b>2.2.</b> <strong>Aménagement</strong> : Existe-t-il des restrictions à la circulation des véhicules professionnels motorisés sur le territoire ?',
        'help' => '(ex : ZFE, ZTL, centre piétonnier, etc.)',
        'formClass' => 'App\Questionnaire\Form\YesNoOtherType',
        'otherOption' => 'Autre (veuillez préciser)',
        'ficheActionRef' => 'diagnostic-reglementaire',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::DIAGNOSTIC,
      'question_input_name' => 'diagnostic-reglementaire-circulation-perimetre',
      'label' => '<b>2.3.</b> Périmètre géographique concerné',
      'attr' => ['data-cond'=>'reg'],
      'formClass' => 'App\Questionnaire\Form\CycloDiagTextType',
      'ficheActionRef' => 'diagnostic-reglementaire',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::DIAGNOSTIC,
      'question_input_name' => 'diagnostic-reglementaire-circulation-horaire',
      'label' => '<b>2.4.</b> Horaires de livraison',
      'attr' => ['data-cond'=>'reg'],
      'formClass' => 'App\Questionnaire\Form\CycloDiagTextType',
      'ficheActionRef' => 'diagnostic-reglementaire',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::DIAGNOSTIC,
      'question_input_name' => 'diagnostic-reglementaire-circulation-motorisation',
      'label' => '<b>2.5.</b> Motorisation (vignette crit’air)',
      'attr' => ['data-cond'=>'reg'],
      'formClass' => 'App\Questionnaire\Form\CycloDiagTextType',
      'ficheActionRef' => 'diagnostic-reglementaire',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::DIAGNOSTIC,
      'question_input_name' => 'diagnostic-reglementaire-circulation-taille_poids',
      'label' => '<b>2.6.</b> Taille/Poids des véhicules',
      'attr' => ['data-cond'=>'reg'],
      'formClass' => 'App\Questionnaire\Form\CycloDiagTextType',
      'ficheActionRef' => 'diagnostic-reglementaire',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::DIAGNOSTIC,
      'question_input_name' => 'diagnostic-foncier-espace_lgtq',
      'label' => '<b>2.7.</b> Foncier: Avez-vous connaissance des espaces logistiques urbains (ELU) sur votre territoire ?',
      'formClass' => 'App\Questionnaire\Form\NoYesSpecifyType',
      'ficheActionRef' => 'Realiser-diagnostic-cyclologistique',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::DIAGNOSTIC,
      'question_input_name' => 'diagnostic-foncier-elu_cyclo',
      'label' => '<b>2.8.</b> <strong>Foncier</strong> : Si oui, combien d’ELU accueillent des activités de cyclologistique ?',
      'attr' => ['data-cond'=>'foncier'],
      'help' => 'NSP si vous ne savez pas',
      'formClass' => 'App\Questionnaire\Form\CycloDiagTextType',
      'ficheActionRef' => 'Realiser-diagnostic-cyclologistique',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::DIAGNOSTIC,
      'question_input_name' => 'diagnostic-foncier-elu_cyclo_conditionnel',
      'label' => '<b>2.9.</b> <strong>Foncier</strong> : Si oui, combien d’ELU pourraient accueillir des activités de cyclologistique ?',
      'attr' => ['data-cond'=>'foncier'],
      'help' => 'NSP si vous ne savez pas',
      'formClass' => 'App\Questionnaire\Form\CycloDiagTextType',
      'ficheActionRef' => 'Realiser-diagnostic-cyclologistique',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::DIAGNOSTIC,
      'sectionHead' => '<strong>Aménagement</strong> : Indice de cyclabilité du territoire<br><span class="questionnaire-help">👉 Merci d’utiliser l’outil de <a href="https://public.tableau.com/app/profile/fabien.commeaux/viz/Cyclabilit_2022_Atlas/Atlas">Vélo et Territoires</a>, pour les EPCI saisissez les éléments de la commune centrale</span>',
      'question_input_name' => 'diagnostic-amenagement-indice_cycla-voirie',
      'label' => '<b>2.10.</b> Cyclabilité de la voirie (en %)',
      'formClass' => 'App\Questionnaire\Form\CycloDiagTextType',
      'ficheActionRef' => 'Developper-amenagements-cyclologistique',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::DIAGNOSTIC,
      'question_input_name' => 'diagnostic-amenagement-indice_cycla-voirie_30',
      'label' => '<b>2.11.</b> Cyclabilité avec voirie 30 km/h (en %)',
      'formClass' => 'App\Questionnaire\Form\CycloDiagTextType',
      'ficheActionRef' => 'Developper-amenagements-cyclologistique',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::DIAGNOSTIC,
      'question_input_name' => 'diagnostic-amenagement-indice_cycla-bidirec',
      'label' => '<b>2.12.</b> Aménagements cyclables bidirectionnels (en m/hab)',
      'formClass' => 'App\Questionnaire\Form\CycloDiagTextType',
      'ficheActionRef' => 'Developper-amenagements-cyclologistique',
    ]));
    $this->add(Question::fromArray([
        'step' => QuestionnaireStep::DIAGNOSTIC,
        'question_input_name' => 'diagnostic-amenagement-indice_circulation_velocargo',
        'label' => '<b>2.13.</b> <strong>Aménagement</strong> : Est-ce que les infrastructures cyclables permettent à des vélos cargos de circuler ?',
        'help' => 'Exemple : absence d’obstacles (pas d’irrégularité de la surface type marches) ou espace suffisant entre des dispositifs anti-véhicules motorisés (au moins 1,5m de large entre deux poteaux)',
        'formClass' => 'App\Questionnaire\Form\YesNoOtherType',
        'otherOption' => 'Autre (veuillez préciser)',
        'ficheActionRef' => 'Developper-amenagements-cyclologistique',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::DIAGNOSTIC,
      'question_input_name' => 'diagnostic-stationnement-aire_livraison',
      'label' => '<b>2.14.</b> <strong>Aménagement</strong> : Existe-t-il des aires de livraisons vélo-cargo ou une signalétique spécifique pour aires de livraison et places de stationnement mixtes, ouvertes aux vélos-cargos ?',
      'formClass' => 'App\Questionnaire\Form\NoYesSpecifyType',
      'ficheActionRef' => 'Developper-amenagements-cyclologistique',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::GOUVERNANCE,
      'question_input_name' => 'gouvernance-portage-deliberation',
      'label' => '<b>3.1.</b> <strong>Portage politique</strong> : L’assemblée d’élus de votre collectivité a-t-elle déjà délibéré sur un sujet de cyclologistique ou une personne élue a-t-elle déjà porté un sujet de cyclologistique ?',
      'formClass' => 'App\Questionnaire\Form\NoYesSpecifyType',
      'ficheActionRef' => 'Engager-les-elus',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::GOUVERNANCE,
      'question_input_name' => 'gouvernance-portage-politique_cyclo',
      'label' => '<b>3.2.</b> <strong>Portage politique</strong> : Existe-t-il une politique de logistique urbaine qui inclut la cyclologistique ?',
      'formClass' => 'App\Questionnaire\Form\NoYesSpecifyType',
      'ficheActionRef' => 'Engager-les-elus',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::GOUVERNANCE,
      'question_input_name' => 'gouvernance-autre-etude_cyclo',
      'label' => '<b>3.3.</b> <strong>Autre</strong> : La collectivité a-t-elle réalisé une étude spécifique sur les activités de cyclologistique sur le territoire ?',
      'formClass' => 'App\Questionnaire\Form\NoYesSpecifyType',
      'ficheActionRef' => 'Realiser-diagnostic-cyclologistique',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::CONCERTATION,
      'question_input_name' => 'concertation-sensibilisation-participation',
      'label' => '<b>4.1.</b> <strong>Portage politique</strong> : La collectivité participe-t-elle à des instances partenariales sur le sujet de la cyclologistique ?',
      'formClass' => 'App\Questionnaire\Form\NoYesSpecifyType',
      'ficheActionRef' => 'Participation-instances-partenariales-programmes',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::CONCERTATION,
      'question_input_name' => 'concertation-sensibilisation-interlud',
      'label' => '<b>4.2.</b> <strong>Portage politique</strong> : La collectivité participe-t-elle au programme Interlud ?',
      'formClass' => 'App\Questionnaire\Form\NoYesSpecifyType',
      'ficheActionRef' => 'Participation-instances-partenariales-programmes',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::CONCERTATION,
      'question_input_name' => 'concertation-sensibilisation-fablog',
      'label' => '<b>4.3.</b> <strong>Portage politique</strong> : La collectivité participe-t-elle à la communauté cyclologistique de la Fabrique de la logistique ?',
      'formClass' => 'App\Questionnaire\Form\NoYesSpecifyType',
      'ficheActionRef' => 'Participation-instances-partenariales-programmes',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::CONCERTATION,
      'question_input_name' => 'concertation-sensibilisation-cyclo_foncier',
      'label' => '<b>4.4.</b> <strong>Portage politique</strong> : Les cyclologisticiens jugent-ils satisfaisants l’accès au foncier sur votre territoire ?',
      'formClass' => 'App\Questionnaire\Form\NoYesSpecifyType',
      'otherOption' => 'La collectivité n’a pas d’échange avec les cyclologisticiens',
      'ficheActionRef' => 'Mettre-a-disposition-foncier-logistique',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::IMPLEMENTATION,
      'question_input_name' => 'implementation-autre-rh-cyclo',
      'label' => '<b>5.1.</b> <strong>Autre</strong> : La collectivité dispose-t-elle d’une ressource humaine dédiée à la logistique urbaine ?',
      'formClass' => 'App\Questionnaire\Form\NoYesSpecifyType',
      'ficheActionRef' => 'Engager-les-elus',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::IMPLEMENTATION,
      'question_input_name' => 'implementation-autre-rh-action_cyclo',
      'label' => '<b>5.2.</b> <strong>Autre</strong> : Si oui, sa feuille de route intègre-t-elle des actions dédiées à la cyclologistique ?',
      'formClass' => 'App\Questionnaire\Form\CycloDiagTextareaType',
      'ficheActionRef' => 'Engager-les-elus',
    ]));
    $this->add(Question::fromArray([
        'step' => QuestionnaireStep::IMPLEMENTATION,
        'sectionHead' => '<strong>Sensibilisation</strong> : La collectivité porte-t-elle des actions de sensibilisation et/ou de formation en faveur d’une cyclologistique responsable :',
        'question_input_name' => 'implementation-sensibilisation-actions-consommateurs',
        'label' => '<b>5.3.</b> À destination des consommateurs ?',
        'formClass' => 'App\Questionnaire\Form\CycloDiagTextType',
        'ficheActionRef' => 'Actions-sensibilisation-formation',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::IMPLEMENTATION,
      'question_input_name' => 'implementation-sensibilisation-actions-commercants',
      'label' => '<b>5.4.</b> À destination des commerçants ?',
      'formClass' => 'App\Questionnaire\Form\CycloDiagTextType',
      'ficheActionRef' => 'Actions-sensibilisation-formation',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::IMPLEMENTATION,
      'question_input_name' => 'implementation-sensibilisation-actions-ent_cyclo',
      'label' => '<b>5.5.</b> À destination des entreprises de la cyclologistique et autres transporteurs ? ',
      'formClass' => 'App\Questionnaire\Form\CycloDiagTextType',
      'ficheActionRef' => 'Actions-sensibilisation-formation',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::IMPLEMENTATION,
      'question_input_name' => 'implementation-sensibilisation-actions-chargeurs',
      'label' => '<b>5.6.</b> À destination des chargeurs ? ',
      'formClass' => 'App\Questionnaire\Form\CycloDiagTextType',
      'ficheActionRef' => 'Actions-sensibilisation-formation',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::IMPLEMENTATION,
      'question_input_name' => 'implementation-sensibilisation-actions-autre',
      'label' => '<b>5.7.</b> Autres exemples ? ',
      'formClass' => 'App\Questionnaire\Form\CycloDiagTextType',
      'ficheActionRef' => 'Actions-sensibilisation-formation',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::IMPLEMENTATION,
      'question_input_name' => 'implementation-sensibilisation-communication',
      'label' => '<b>5.8.</b> <strong>Sensibilisation</strong> : La collectivité communique-t-elle sur les actions mises en place en faveur de la cyclologistique ? ',
      'formClass' => 'App\Questionnaire\Form\NoYesSpecifyType',
      'otherOption' => 'La collectivité ne mène pas d’action en faveur de la cyclologistique',
      'ficheActionRef' => 'Actions-sensibilisation-formation',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::IMPLEMENTATION,
      'question_input_name' => 'implementation-autre-colis_activ',
      'label' => '<b>5.9.</b> <strong>Autre</strong> : La collectivité participe-t-elle au programme Colis’Activ ?',
      'formClass' => 'App\Questionnaire\Form\NoYesSpecifyType',
      'ficheActionRef' => 'Participation-instances-partenariales-programmes',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::IMPLEMENTATION,
      'question_input_name' => 'implementation-autre-suivi_cyclo',
      'label' => '<b>5.10.</b> <strong>Autre</strong> : La collectivité met-elle en place un dispositif qui lui permet de suivre l’évolution des activités de cyclologistique sur le territoire ?',
      'formClass' => 'App\Questionnaire\Form\NoYesSpecifyType',
      'ficheActionRef' => 'Realiser-diagnostic-cyclologistique',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::IMPLEMENTATION,
      'question_input_name' => 'implementation-autre-clause_cyclo',
      'label' => '<b>5.11.</b> <strong>Portage Politique</strong> : La collectivité intègre-t-elle une clause spécifique à la cyclologistique dans ses marchés publics ?',
      'formClass' => 'App\Questionnaire\Form\NoYesSpecifyType',
      'ficheActionRef' => 'Marches-publics',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::IMPLEMENTATION,
      'question_input_name' => 'implementation-foncier-facilite',
      'label' => '<b>5.12.</b> <strong>Foncier</strong> : La collectivité facilite-elle l’utilisation du foncier public ou privé pour les activités de cyclologistique ?',
      'formClass' => 'App\Questionnaire\Form\NoYesSpecifyType',
      'ficheActionRef' => 'Mettre-a-disposition-foncier-logistique',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::IMPLEMENTATION,
      'question_input_name' => 'implementation-foncier-inte_urbanisme',
      'label' => '<b>5.13.</b> <strong>Foncier</strong> : La cyclologistique est-elle intégrée aux documents d’urbanisme (SCoT, PLU) permettant de lui réserver du foncier ?',
      'formClass' => 'App\Questionnaire\Form\NoYesSpecifyType',
      'ficheActionRef' => 'Reserver-foncier-logistique',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::IMPLEMENTATION,
      'question_input_name' => 'implementation-foncier-investissement-dispositifs',
      'label' => '<b>5.14.</b> <strong>Aide à l’investissement</strong> : La collectivité a-t-elle des dispositifs d’aide à l’installation/à l’investissement pour la cyclologistique ?',
      'help' => '(achat vélo-cargo pour les professionnels, chambre froide, logiciels, maintenance…)',
      'formClass' => 'App\Questionnaire\Form\NoYesSpecifyType',
      'ficheActionRef' => 'Dispositifs-aide-installation-investissement',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::IMPLEMENTATION,
      'question_input_name' => 'implementation-foncier-investissement-location',
      'label' => '<b>5.15.</b> <strong>Aide à l’investissement</strong> : La collectivité propose-t-elle des locations longue durée de vélos-cargo pour les entreprises dans le cadre d’une activité de cyclologistique ?',
      'formClass' => 'App\Questionnaire\Form\NoYesSpecifyType',
      'ficheActionRef' => 'Dispositifs-mise-a-disposition-vélos-cargos',
    ]));
    $this->add(Question::fromArray([
      'step' => QuestionnaireStep::IMPLEMENTATION,
      'question_input_name' => 'implementation-foncier-investissement-experimentation',
      'label' => '<b>5.16.</b> <strong>Aide à l’investissement</strong> : La collectivité propose t’elle des expérimentations courte durée de mise à disposition de vélos-cargo pour les entreprises dans le cadre d’une activité de cyclologistique ?',
      'formClass' => 'App\Questionnaire\Form\NoYesSpecifyType',
      'ficheActionRef' => 'Dispositifs-mise-a-disposition-vélos-cargos',
    ]));

    $this->questionCollection = new \ArrayObject($this->questions);

    unset($this->questions);
  }

  public function count()
  {

    return $this->questionCollection->count();
  }

  public function getQuestionsForAllSteps()
  {

    return $this->questionCollection;
  }

  public function getQuestionsForStep(QuestionnaireStep $step)
  {
    $this->step = $step;

    return $this->getIterator();
  }

//  https://dev.to/yactouat/introducing-php-custom-collections-with-the-iteratoraggregate-interface-1469
// file:///home/aleth/Documents/TechDoc/PHP_Generators/PHP iterators and generators_ get started with the `ArrayIterator` and the `FilterIterator` classes.html
  public function getIterator(): \Traversable|\FilterIterator
  {

    return new class($this->questionCollection, $this->step) extends \FilterIterator
    {
          private $step;
          public function __construct(\ArrayObject $questionCollection, QuestionnaireStep $step) {
              parent::__construct($questionCollection->getIterator());
              $this->step = $step;
          }

          public function accept(): bool {
              return $this->current()->getStep() === $this->step;
          }
    };
  }

  public function getQuestion(QuestionInputName $questionInputName)
  {
    return $this->questionCollection[(string)$questionInputName];
  }

  private function add(Question $question)
  {
    $this->questions[(string)$question->getInputName()] = $question;
  }

}
