<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Repository;

use App\Entity\CommuneEPCI;
use App\Entity\LatestCollectiviteDTO;
use Doctrine\DBAL\Connection;

class CommuneEPCIRepository
{
  public function __construct(
    private Connection $connection,
  )
  {}

    public function save(CommuneEPCI $communeEPCI) {
        $record = $communeEPCI->toDatabaseRecord();
        $this->connection->insert($communeEPCI::tableName(), $record, [
            \PDO::PARAM_STR,
            \PDO::PARAM_STR,
            \PDO::PARAM_INT,
        ]);
    }
    public function delete(int $id) {
        return $this->connection->delete('communeEPCI', [
            'id' => $id
        ], [
            \PDO::PARAM_INT
        ]);
    }

    public function lastInsertedId(){
      return $this->connection->lastInsertId();
    }

    public function getMeta(int $id)
    {
        return $this->connection->fetchAssociative(<<<SQL
            SELECT name, logoUrl, publicPageSlug, publicPageLastVersion
            FROM communeEPCI 
            WHERE id = ?
SQL,
            [$id], [\PDO::PARAM_INT]
        );
    }

    public function getMetaBySLug(string $slug)
    {

        return $this->connection->fetchAssociative(<<<SQL
            SELECT name, logoUrl, publicPageSlug, publicPageLastVersion
            FROM communeEPCI 
            WHERE publicPageSlug = ?
SQL,
            [$slug], [\PDO::PARAM_STR]
        );
    }

    public function updateLogoUrl(string $logoUrl, array $criteria)
    {
        $communeEPCI_id = $this->connection->fetchOne(
            " SELECT communeEPCI_id FROM diagnosticCycloDiag WHERE uuid = ? ", [
            $criteria['diagUuid']
        ], [
            \PDO::PARAM_STR
        ]);

        $this->connection->executeStatement(
            " UPDATE communeEPCI SET logoUrl = ? WHERE  id = ?", [
            $logoUrl, $communeEPCI_id
        ], [
            \PDO::PARAM_STR, \PDO::PARAM_INT
        ]);
    }

    public function findOne(int $communeEPCI_id): CommuneEPCI
    {
        $row = $this->connection->fetchAssociative(
            "SELECT id, name, region, version 
        FROM communeEPCI
        WHERE id=?", [$communeEPCI_id], [\PDO::PARAM_INT]);
        return CommuneEPCI::fromDatabaseRecord($row);
    }

    public function findLastThree(): array
    {
        $list = [];
        $sql = "SELECT name, logoUrl FROM communeEPCI 
        ORDER BY createdAt DESC LIMIT 3 ";

        foreach ($this->connection->fetchAllAssociative($sql) as $row) {
            $list[] = LatestCollectiviteDTO::fromDatabaseRecord($row);
        }

        return $list;
    }

    public function findAll(): array
    {
        $list = [];
        $sql = "SELECT name, logoUrl FROM communeEPCI 
        ORDER BY createdAt DESC";

        foreach ($this->connection->fetchAllAssociative($sql) as $row) {
            $list[] = LatestCollectiviteDTO::fromDatabaseRecord($row);
        }

        return $list;
    }

    public function nbrUsersForCollectivite(int $userId)
    {
        return $this->connection->fetchAssociative(<<<SQL
        SELECT COUNT(u.id) AS nbrUser, c.id as collectiviteId
        FROM communeEPCI c 
            LEFT JOIN user u ON u.communeEPCI_id = c.id 
        GROUP BY c.id 
        HAVING ? IN (GROUP_CONCAT(u.id));
SQL,[
    $userId
        ], [
            \PDO::PARAM_INT
        ]);
    }

    public function getIdByPublicId(string $publicId)
    {
        return $this->connection->fetchOne("
        SELECT id FROM communeEPCI WHERE publicId = ? ", [
            $publicId
        ], [
           \PDO::PARAM_STR
        ]);
    }
    public function getPublicIdById(int $id): ?string
    {
        return $this->connection->fetchOne("
        SELECT publicId FROM communeEPCI WHERE id = ? ", [
            $id
        ], [
           \PDO::PARAM_INT
        ]);
    }

    public function updatePublicId(int $collectiviteId, string $collectivitePublicId)
    {
        $this->connection->executeStatement("
        UPDATE communeEPCI SET publicId = ? WHERE id = ? ", [
            $collectivitePublicId, $collectiviteId
        ], [
            \PDO::PARAM_STR, \PDO::PARAM_INT
        ]);
    }

    public function updatePublicPageSlug(int $collectiviteId)
    {
        $collectivite = $this->findOne($collectiviteId);
        $publicPageSlug = $collectivite->sluggedName();
        $this->connection->executeStatement(
            " UPDATE communeEPCI SET publicPageSlug = ? WHERE  id = ?", [
            $publicPageSlug, $collectiviteId
        ], [
            \PDO::PARAM_STR, \PDO::PARAM_INT
        ]);
    }

    public function setPageLastVersion(int $collectiviteId, int $publicPageVersion)
    {
        $this->connection->update('communeEPCI', [
            'publicPageLastVersion' => $publicPageVersion
        ], [
            'id' => $collectiviteId
        ], [
            \PDO::PARAM_INT,
            \PDO::PARAM_INT,
        ]);
    }



}
