<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Repository;

use App\Entity\DiagnosticCycloDiag;
use App\ViewDTO\CommuneEPCIDiagList;
use App\ViewDTO\DiagnosticForDashboard;
use Doctrine\DBAL\Connection;

class DiagnosticCycloDiagRepository
{
    public function __construct(
        private Connection $connection,
    )
    {
    }

    public function save(DiagnosticCycloDiag $diagnosticICL)
    {
        $this->connection->insert(
            $diagnosticICL::tableName(),
            $diagnosticICL->toDbRecord(),
            $diagnosticICL::params()
        );
    }

    public function update(DiagnosticCycloDiag $diagnosticICL): void
    {
        $this->connection->update(
            $diagnosticICL::tableName(),
            $diagnosticICL->toDbRecord(),
            $diagnosticICL->identifier(),
            $diagnosticICL::params()
        );
    }

    public function updatedAt(DiagnosticCycloDiag|string $diagnosticICLOrUuid): void
    {
        if ($diagnosticICLOrUuid instanceof DiagnosticCycloDiag) {
            $identifier = $diagnosticICLOrUuid->identifier();
        } else {
            $identifier = [
                'uuid' => $diagnosticICLOrUuid
            ];
        }

        $this->connection->update(
            DiagnosticCycloDiag::tableName(),
            [
                'updatedAt' => (new \DateTimeImmutable())->format('Y-m-d H:i:s'),
            ],
            $identifier,
            [
                \PDO::PARAM_STR,
            ]
        );
    }

    // Added for testing purpose
    public function setUpdated(DiagnosticCycloDiag $diagnosticCycloDiag, \DateTimeImmutable $date): void
    {
        $this->connection->update(
            DiagnosticCycloDiag::tableName(),
            [
                'updatedAt' => $date->format('Y-m-d H:i:s'),
            ],
            $diagnosticCycloDiag->identifier(),
            [
                \PDO::PARAM_STR,
                \PDO::PARAM_STR,
            ]
        );
    }

    public function delete(string $uuid) {
        return $this->connection->delete(DiagnosticCycloDiag::tableName(), [
            'uuid' => $uuid
        ], [
            \PDO::PARAM_STR
        ]);
    }

    public function findOne(string $diagUuid)
    {
        $table = DiagnosticCycloDiag::tableName();
        $row = $this->connection->fetchAssociative(<<<SQL
        SELECT uuid, communeEPCI_id, ownerId, duplicatedFrom, name, version
        FROM $table
        WHERE uuid = ?
SQL, [
            $diagUuid
        ], [
                \PDO::PARAM_STR
            ]
        );
        if ($row !== false) {
            return DiagnosticCycloDiag::fromDbRecord($row);
        }

        return [];
    }

    public function findLatestUuidForCommune(int $communeEPCI_id): string
    {
        $table = DiagnosticCycloDiag::tableName();
        return $this->connection->fetchOne(
            "SELECT uuid FROM $table WHERE communeEPCI_id= ? ORDER BY createdAt DESC LIMIT 1", [
                $communeEPCI_id
            ], [
                \PDO::PARAM_INT
            ]
        );
    }

    public function findUuidForUser(int $userId): string
    {
        $table = DiagnosticCycloDiag::tableName();
        return $this->connection->fetchOne(
            "SELECT uuid FROM $table WHERE ownerId = ? ORDER BY createdAt DESC LIMIT 1", [
                $userId
            ], [
                \PDO::PARAM_INT
            ]
        );
    }

    public function findCommuneIdByUuid(string $uuid): int
    {
        $table = DiagnosticCycloDiag::tableName();
        return $this->connection->fetchOne(<<<SQL
    SELECT communeEPCI_id FROM $table WHERE uuid = ?
SQL, [$uuid], [\PDO::PARAM_STR]);
    }

    public function findAll(int $communeEPCI_id)
    {
        $table = DiagnosticCycloDiag::tableName();
        $list = [];

        $rows = $this->connection->fetchAllAssociativeIndexed(<<<SQL
        SELECT  uuid, name, duplicatedFrom ,createdAt, updatedAt 
        FROM $table
        WHERE communeEPCI_id = ?
        ORDER BY updatedAt DESC
SQL, [
            $communeEPCI_id
        ], [
                \PDO::PARAM_INT
            ]
        );

        foreach ($rows as $uuid => $row) {
            $list[$uuid] = CommuneEPCIDiagList::fromRow($row);
        }

        return $list;
    }

    public function getAllWithStepCompletion(): array
    {
        $table = DiagnosticCycloDiag::tableName();

        $rows = $this->connection->fetchAllAssociative(<<<SQL

            SELECT uuid, diag.name as 'diagName', comm.name as 'commune_EPCI', comm.region as region, u.fullName as 'ownerName', comps as 'completionsByStep', diag.updatedAt as 'lastUpdated', duplicatedFrom
            FROM $table diag
            LEFT JOIN communeEPCI comm
              ON diag.communeEPCI_id = comm.id
            LEFT JOIN user u
              ON u.id = diag.ownerId
            LEFT JOIN (
                SELECT diagUuid, JSON_ARRAYAGG(JSON_OBJECT('step',step, 'comp', c)) as comps
                FROM (
                    SELECT diagUuid, step as step, count(step) as c 
                    FROM reponseCycloDiag 
                    GROUP BY diagUuid, step ORDER BY FIELD(step, 'CONTEXTE', 'DIAGNOSTIC', 'GOUVERNANCE', 'CONCERTATION', 'IMPLEMENTATION') 
                    ) as comp group by diagUuid
                ) as dcomp
                ON diag.uuid = dcomp.diagUuid
            ORDER BY lastUpdated DESC
SQL);
//        dd($rows);

        return array_map(function($row){
            return DiagnosticForDashboard::fromDbRecord($row);
        }, $rows);

    }
}
