<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Repository;

use App\Entity\FicheAction;
use App\ViewDTO\FicheActionMemoDTO;
use App\ViewDTO\FicheActionMenuLink;
use App\ViewDTO\FicheActionPreviewDTO;
use Doctrine\DBAL\Connection;
use Doctrine\DBAL\Exception\UniqueConstraintViolationException;
use Doctrine\DBAL\Exception;
use Psr\Log\LoggerInterface;

class FicheActionRepository
{
    public function __construct(
        private Connection $connection,
        private LoggerInterface $logger,
    )
    {
    }

    public function findFicheActionPreviewByRefs(array $ficheActionRefs): array
    {
        $table = FicheActionPreviewDTO::tableName();
        $rows = $this->connection->fetchAllAssociative(<<<SQL
        SELECT reference, viewOrder, title, slug, theme, preview 
        FROM $table
           WHERE reference IN (:fas)
           AND archivedAt IS NULL
           ORDER BY FIELD(reference, :fas)
SQL, [
            'fas' => $ficheActionRefs,
        ], [
            'fas' => \Doctrine\DBAL\ArrayParameterType::STRING,
        ]);

       $ficheActionPreview = array_map(function (array $row){
            return FicheActionPreviewDTO::fromRow($row);
        }, $rows);

        return $ficheActionPreview;
    }

    public function allFicheActionPreview(): array
    {
        $table = FicheActionPreviewDTO::tableName();
        $rows = $this->connection->fetchAllAssociative(<<<SQL
        SELECT reference, viewOrder, title, slug, theme, preview 
        FROM $table
        WHERE archivedAt IS NULL
           ORDER BY viewOrder
SQL);

       $ficheActions = array_map(function (array $row){
            return FicheActionPreviewDTO::fromRow($row);
        }, $rows);

        return $ficheActions;
    }

    public function allIndexedFicheActionMemo()
    {
        $table = FicheActionPreviewDTO::tableName();
        $rows = $this->connection->fetchAllAssociativeIndexed(<<<SQL
        SELECT reference, title, slug, preview 
        FROM $table
        WHERE archivedAt IS NULL
           ORDER BY viewOrder
SQL);

        foreach ($rows as $reference => $row) {
            $ficheActions[$reference] = FicheActionMemoDTO::fromRow($row);
        }

        return $ficheActions;
    }

    public function allFicheActionForMenu(): array
    {
        $table = FicheActionPreviewDTO::tableName();
        $rows = $this->connection->fetchAllAssociative(<<<SQL
        SELECT title as label, slug FROM $table
        WHERE content != '' AND archivedAt IS NULL
        ORDER BY viewOrder
SQL);

        $ficheActionSubMenu = array_map(function (array $row){
            return FicheActionMenuLink::fromRow($row);
        }, $rows);

        return $ficheActionSubMenu;
    }

    public function findOne(string $reference): FicheAction
    {
        $table = FicheActionPreviewDTO::tableName();

        $row = $this->connection->fetchAssociative(<<<SQL
        SELECT * FROM $table WHERE reference = ?
SQL, [
            $reference], [
            \PDO::PARAM_STR
        ]);

        return FicheAction::fromDB($row);
    }

    public function isReferenceInUse(string $reference): bool
    {
        $table = FicheActionPreviewDTO::tableName();

        return (bool)$this->connection->fetchOne("
            SELECT COUNT(*) FROM $table WHERE reference = ?", [
            $reference
        ], [
            \PDO::PARAM_STR
        ]);
    }

    public function allFicheActionContents(): array
    {
        $table = FicheActionPreviewDTO::tableName();
        return $this->connection->fetchAllAssociative(<<<SQL
        SELECT slug, theme, title, content FROM $table
        WHERE content != '' AND archivedAt IS NULL
        ORDER BY viewOrder
SQL);

    }

    public function save(FicheAction $ficheAction)
    {
        try {
        $this->connection->insert(FicheAction::tableName(), $ficheAction->toDBRecord(), FicheAction::params());
        } catch (UniqueConstraintViolationException $e) { // this is an update
            try {
                $this->update($ficheAction);
            } catch (ConcurrentUpdateOccurred $e) {

                $this->logger->error($e->getMessage());
                // rethrow error to be captured in Controller
                throw new \Exception('Cette fiche a été modifiée simultanément par une autre personne. Veuillez recharger la page et réessayer');
            }
        } catch (Exception $e) {
            $this->logger->error($e->getMessage());
            throw new \Exception('L’enregistrement a échoué. Veuillez réssayer');
        }
    }

    private function update(FicheAction $ficheAction)
    {
        $table = FicheAction::tableName();
        $where = [];
        $values = [];
        $sql = 'SELECT version FROM ' . $table . ' WHERE  reference = ?' ;
        $versionInDB = (int)$this->connection->fetchOne($sql, [
            $ficheAction->reference
        ],[
            \PDO::PARAM_STR]);

        $data = $ficheAction->toDBRecord(); // version is incremented here
        $responseVersion = $ficheAction->version();

        if ($versionInDB >= $responseVersion) {
            throw  ConcurrentUpdateOccurred::ofEntity($ficheAction);
        }

        try {
            $this->connection->update(
                $table,
                 $data + [
                    'updatedAt' => (new \DateTimeImmutable())->format('Y-m-d H:i:s')
                ],
                $ficheAction->identifier(),
                $ficheAction::params()
            );
        } catch (Exception $e) {
            $this->logger->error($e->getMessage());
            throw new \Exception('L’enregistrement a échoué. Veuillez recharger la page et réssayer');
        }
    }

    public function getMaxViewOrder(): int
    {
        return $this->connection->fetchOne("
            SELECT MAX(viewOrder) FROM ficheAction ");
    }

    public function saveOrder(array $orderedReferences)
    {
        $stmt = $this->connection->prepare("
            UPDATE ficheAction SET viewOrder = ? WHERE reference = ?");

        foreach ($orderedReferences as $index => $reference) {
            $stmt->bindValue(1, $index);
            $stmt->bindValue(2, $reference, \PDO::PARAM_STR);
            $stmt->executeStatement();
        }
    }

}
