<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Repository;

use App\Entity\InviteConsultantToken;
use App\Entity\ResponseCycloDiag_DTO;
use App\ViewDTO\ConsultantQuestion;
use App\ViewDTO\QuestionForConsultant;
use Doctrine\DBAL\Connection;

class InviteTokenRepository
{
    public function __construct(
        private readonly Connection $connection,
    )
    {
    }

    public function save(InviteConsultantToken $inviteConsultantToken)
    {
        $this->connection->insert(
            InviteConsultantToken::tableName(),
            $inviteConsultantToken->toDBRecord(),
            $inviteConsultantToken::params(),
        );
    }

    public function getAll(): array
    {
        $table = InviteConsultantToken::tableName();
        return $this->connection->fetchAllAssociative("
        SELECT * from $table");
    }

    public function findOneBySelector(string $selector): InviteConsultantToken
    {
        $this->removeExpiredInviteToken();

        $table = InviteConsultantToken::tableName();
        $row = $this->connection->fetchAssociative(<<<SQL
        SELECT selector, userId, diagUuid, questionInputName, expiresAt, hashedVerifier
        FROM $table
        WHERE selector = ? AND NOW() < expiresAt
SQL, [
            $selector
        ], [
            \PDO::PARAM_STR
        ]);
        return InviteConsultantToken::fromDB($row);
    }

    public function findQuestionsFor(int $userId, string $diagUuid): ?array
    {
        $table = InviteConsultantToken::tableName();

        $rows = $this->connection->fetchAllKeyValue(<<<SQL
            SELECT selector, questionInputName
            FROM $table
            WHERE userId = ? AND diagUuid = ? AND expiresAt > NOW()
SQL, [
            $userId, $diagUuid], [
            \PDO::PARAM_INT, \PDO::PARAM_STR
        ]);

        if (!empty($rows)) {
            $res = [];
            foreach (array_values($rows) as $row) {
                $res[] = (string)QuestionForConsultant::fromDB($row)->questionInputName;
            }

            return $res;
        }

        return null;
    }

    public function findAllConsultantsAndQuestions(string $diagUuid): array
    {
        $list = [];
        $t = InviteConsultantToken::tableName();
        $r = ResponseCycloDiag_DTO::tableName();

        $rows = $this->connection->fetchAllAssociative(<<<SQL
        SELECT u.fullName, u.email, t.questionInputName, r.updatedAt as respondedAt
               FROM $t t
                LEFT JOIN user u ON u.id = t.userId
                LEFT JOIN $r r ON r.inputName = t.questionInputName
        WHERE t.diagUuid = ? 
        ORDER BY u.createdAt
SQL, [
            $diagUuid
        ], [\PDO::PARAM_STR]);

        return array_map(function ($row) {

                return ConsultantQuestion::fromRow($row);
        }, $rows);
    }

    public function removeInviteToken(InviteConsultantToken $inviteToken): void
    {
        $table = InviteConsultantToken::tableName();

        $this->connection->delete($table, $inviteToken->identifier(), [
            \PDO::PARAM_INT,
            \PDO::PARAM_STR,
            \PDO::PARAM_STR,
        ]);
    }

    public function removeInviteTokenForUser(int $userId): void
    {

        $this->connection->createQueryBuilder()
            ->delete(InviteConsultantToken::tableName())
            ->where('userId = :userId')
            ->setParameter('userId', $userId)
            ->executeQuery();
    }

    private function removeExpiredInviteToken(): void
    {
        $time = (new \DateTimeImmutable('-3 month'))->format('Y-m-d H:i:s');

        $this->connection->createQueryBuilder()
            ->delete(InviteConsultantToken::tableName())
            ->where('expiresAt <= :time')
            ->setParameter('time', $time)
            ->executeQuery();
    }

}
