<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Repository;

use App\Entity\ResponseCycloDiag_DTO;
use App\Questionnaire\QuestionnaireCycloDiag;
use Doctrine\DBAL\Connection;
use Doctrine\DBAL\Exception;
use Doctrine\DBAL\Exception\UniqueConstraintViolationException;
use Psr\Log\LoggerInterface;

class ResponseCycloDiagRepository
{
    public function __construct(
        private Connection      $connection,
        private LoggerInterface $logger,
    )
    {}

    public function save(ResponseCycloDiag_DTO $responseICL)
    {
        try {
            $this->connection->insert(
                $responseICL::tableName(),
                $responseICL->toDBRecord(),
                $responseICL::params()
            );
        } catch (UniqueConstraintViolationException $e) { // this is an update
            try {
                $this->update($responseICL);
            } catch (ConcurrentUpdateOccurred $e) {

                $this->logger->error($e->getMessage());
                // rethrow error to be captured in Controller
                throw new \Exception('Cette question a été modifiée simultanément par une autre personne. Veuillez recharger la page et réessayer');
            }
        } catch (Exception $e) {
            $this->logger->error($e->getMessage());
            throw new \Exception('L’enregistrement a échoué. Veuillez réssayer');
        }
    }

    public function update(ResponseCycloDiag_DTO $responseICL)
    {
        $table = ResponseCycloDiag_DTO::tableName();
        $where = [];
        $values = [];
        foreach ($responseICL->identifier() as $col => $val) {
            $where[] = $col . ' = ?';
            $values[] = $val;
        }
        $sql = 'SELECT version FROM ' . $table . ' WHERE ' . implode(' AND ', $where);
        $versionInDB = (int)$this->connection->fetchOne($sql, $values);

        $data = $responseICL->toDBRecord(); // version is incremented here
        $responseVersion = $responseICL->version();

        if ($versionInDB >= $responseVersion) {
            throw  ConcurrentUpdateOccurred::ofEntity($responseICL);
        }

        try {
            $this->connection->update(
                ResponseCycloDiag_DTO::tableName(),
                 $data + [
                    'updatedAt' => (new \DateTimeImmutable())->format('Y-m-d H:i:s')
                ],
                $responseICL->identifier(),
                $responseICL::params()
            );
        } catch (Exception $e) {
            $this->logger->error($e->getMessage());
            throw new \Exception('L’enregistrement a échoué. Veuillez recharger la page et réssayer');
        }
    }

    public function delete(string $diagUuid) {
        return $this->connection->delete(ResponseCycloDiag_DTO::tableName(), [
            'diagUuid' => $diagUuid
        ], [
            \PDO::PARAM_STR
        ]);
    }

    public function findOne(array $identifier) {
        $table = ResponseCycloDiag_DTO::tableName();
        $where = [];
        $values = [];
        foreach ($identifier as $col => $val) {
            $where[] = $col . ' = ?';
            $values[] = $val;
        }
        $row = $this->connection->fetchAssociative(
            'SELECT * FROM '.$table.' WHERE '.implode(' AND ', $where),
            $values
        );

        return ResponseCycloDiag_DTO::fromDBRecord($row);
    }

    public function getValuesVersionForDiag(string $diagUuid)
    {
        $table = ResponseCycloDiag_DTO::tableName();
        return $this->connection->fetchAllAssociativeIndexed(<<<SQL
        SELECT inputName, value, other, version
        FROM $table
        WHERE diagUuid = ?
SQL, [
            $diagUuid
        ], [
                \PDO::PARAM_STR
            ]
        );
    }

    public function getCompletionForDiag(string $diagUuid): array
    {
        $table = ResponseCycloDiag_DTO::tableName();
        return $this->connection->fetchAllKeyValue(<<<SQL
        SELECT step, COUNT(*) AS completion 
        FROM $table  
        WHERE diagUuid = ?
        GROUP BY step ;
SQL, [
            $diagUuid
        ], [
            \PDO::PARAM_STR
        ]);
    }

    public function getExportedResponsesFromDiag(string $diagUuid): array
    {

        $rows = $this->connection->fetchAllAssociativeIndexed(<<<SQL
        SELECT inputName, value, other, r.updatedAt, collegues
        FROM reponseCycloDiag r
        LEFT JOIN (
            SELECT questionInputName, GROUP_CONCAT(fullName SEPARATOR '\r') AS collegues
            FROM inviteConsultantToken i
            LEFT JOIN user u
                ON u.id = i.userId
            WHERE diagUuid = ?
            GROUP BY questionInputName
        ) AS q
        ON r.inputName = q.questionInputName
        WHERE diagUuid = ?
SQL,
            [
                $diagUuid, $diagUuid
            ], [
                \PDO::PARAM_STR, \PDO::PARAM_STR
            ]);

        $result = [];

        foreach ((new QuestionnaireCycloDiag())->getQuestionsForAllSteps() as $question) {
            $questionInputName = (string)$question->getInputName();
            $reponse = (isset($rows[$questionInputName])) ? $rows[$questionInputName]['value'] . "\r" . $rows[$questionInputName]['other'] : '';

            // $result[$questionInputName] = [ for debug purpose
            $result[] = [
                'question' => $question->formatLabel(),
                'réponse' => $reponse,
                'dernière mise à jour' => $rows[$questionInputName]['updatedAt'] ?? null,
                'collègues sollicités' => $rows[$questionInputName]['collegues'] ?? null,
            ];
        }

        return $result;
    }

    public function getCountForDiag(string $diagUuid)
    {
        return $this->connection->fetchOne("SELECT count(*) FROM reponseCycloDiag WHERE diagUuid = ?", [
            $diagUuid
        ], [
            \PDO::PARAM_STR
        ]);
    }

}
