<?php
/*
 * Copyright 2024 Aleth Gueguen <conseil@alethgueguen.com>
 *  This file is part of CycloDiag.
 *
 * CycloDiag is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * CycloDiag is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with CycloDiag. If not, see <https://www.gnu.org/licenses/>
 *
 */

namespace App\Repository;

use App\Questionnaire\QuestionnaireCycloDiag;
use App\Entity\BeneficiaireAdminReminderDTO;
use App\Entity\CollegueReminderDTO;
use App\Config\ReminderInterval;
use Doctrine\DBAL\Connection;

class UserReminderRepository
{

    private int $questionsCount;

    public function __construct(
        private Connection $connection,
    )
    {
        $this->questionsCount = (new QuestionnaireCycloDiag())->count();
    }

    public function findBeneficiaireAdminToSendReminder()
    {
        /*
        Beneficiaire Admin users

        SELECT fullName , email
        FROM user u
        WHERE json_contains(roles, '"ROLE_BENEFICIAIRE_ADMIN"')


        find latest diag in flight for user

        SELECT fullName , email, d.uuid, d.updatedAt as lastResponse
        LEFT JOIN diagnosticCycloDiag d
          ON u.communeEPCI_id=d.communeEPCI_id
        WHERE
          AND d.uuid = (
            SELECT uuid
            FROM diagnosticCycloDiag z
            WHERE z.communeEPCI_id = u.communeEPCI_id
            ORDER BY createdAt DESC LIMIT 1)


          Find responseCount done by diag

          SELECT fullName , email, d.uuid, responseCount,
          LEFT JOIN (
            SELECT COUNT(*) AS responseCount, diagUuid
              FROM reponseCycloDiag
              GROUP BY diagUuid
              ) r ON r.diagUuid = d.uuid


          if all responses are done, do not select this user
          AND responseCount < {$this->questionsCount}


          lastResponse is more than 10 days and no reminder sent
          or lastResponse is more than 30 days and 1 reminder already sent -> on 2nd reminder l.sent will be 'J+30'

        AND (
          (DATEDIFF(NOW(), d.updatedAt) >= 10 AND l.sent IS NULL)
          OR (DATEDIFF(NOW(), d.updatedAt) >= 30   AND l.sent  = 'J+10')
          )
         */

        $j10 = ReminderInterval::J10->value;
        $sql = <<<SQL
      SELECT u.id AS userId, fullName , email, d.uuid AS diagUuid, d.updatedAt AS lastResponse, responseCount, l.sent
      FROM user u 
      LEFT JOIN diagnosticCycloDiag d 
        ON u.communeEPCI_id=d.communeEPCI_id
      LEFT JOIN (
        SELECT COUNT(*) AS responseCount, diagUuid
          FROM reponseCycloDiag
          GROUP BY diagUuid
          ) r ON r.diagUuid = d.uuid 
      LEFT JOIN reminderLog l ON l.userId = u.id
      WHERE json_contains(roles, '"ROLE_BENEFICIAIRE_ADMIN"') 
        AND d.uuid = (
          SELECT uuid
          FROM diagnosticCycloDiag z
          WHERE z.communeEPCI_id = u.communeEPCI_id
          ORDER BY createdAt DESC LIMIT 1)
      AND responseCount < {$this->questionsCount}
      AND (
          l.userId IS NULL 
        OR 
        (DATEDIFF(NOW(), d.updatedAt) >= 10 AND l.sent IS NULL) 
        OR (DATEDIFF(NOW(), d.updatedAt) >= 30   AND l.sent = '$j10')
        )
SQL;
        $userReminders = [];

        foreach ($this->connection->fetchAllAssociative($sql) as $row) {
            $userReminders[] = BeneficiaireAdminReminderDTO::fromDatabaseRecord($row);
        }

        return $userReminders;
    }


    public function findCollegueToSendReminder()
    {

        /*
          find aggregated responses by diag and userId from user invited

          (
          SELECT u.id as userId, fullName, email, i.diagUuid, json_arrayagg(questionInputName) as qsToRespond, count(questionInputName) as qsToRespondCount, count(r.inputName) as qsRespondedCount , ifnull(max(r.updatedAt), i.createdAt) as lastResponse
          FROM inviteConsultantToken i
          LEFT JOIN user u ON u.id = i.userId
          LEFT JOIN reponseCycloDiag r ON r.diagUuid = i.diagUuid AND r.inputName = i.questionInputName
          WHERE i.diagUuid = (
                  SELECT uuid
                  FROM diagnosticCycloDiag z
                  WHERE z.communeEPCI_id = u.communeEPCI_id
                  ORDER BY createdAt DESC LIMIT 1)
          GROUP BY i.diagUuid, i.userId
            HAVING qsRespondedCount < qsToRespondCount
          ) rs

          if qsRespondedCount = qsToRespondCount do not select this user
                        HAVING qsRespondedCount < qsToRespondCount

         */

        $j7 = ReminderInterval::J7->value;
//    dd($j7);
        $sql = <<<SQL
      SELECT rs.userId, fullName, email, diagUuid, lastResponse, qsToRespond, qsToRespondCount, qsRespondedCount, l.sent
      FROM (
        SELECT
          u.id AS userId,
          fullName,
          email,
          i.diagUuid,
          JSON_ARRAYAGG(questionInputName) as qsToRespond,
          COUNT(questionInputName) AS qsToRespondCount,
          COUNT(r.inputName) AS qsRespondedCount ,
          IFNULL(MAX(r.updatedAt),
          i.createdAt) AS lastResponse
        FROM inviteConsultantToken i
        LEFT JOIN user u ON u.id = i.userId
        LEFT JOIN reponseCycloDiag r ON r.diagUuid = i.diagUuid AND r.inputName = i.questionInputName
        WHERE i.diagUuid = (
                SELECT uuid
                FROM diagnosticCycloDiag z
                WHERE z.communeEPCI_id = u.communeEPCI_id
                ORDER BY createdAt DESC LIMIT 1)
        GROUP BY i.diagUuid, i.userId
          HAVING qsRespondedCount < qsToRespondCount
        ) rs
      LEFT OUTER JOIN reminderLog l ON l.userId = rs.userId
      WHERE l.userId IS NULL 
        OR (
          (DATEDIFF(NOW(), lastResponse) >= 7 AND l.sent IS NULL)
          OR (DATEDIFF(NOW(), lastResponse) >= 14   AND l.sent = '$j7'))
SQL;

        $userReminders = [];

        foreach ($this->connection->fetchAllAssociative($sql) as $row) {
            $userReminders[] = CollegueReminderDTO::fromDatabaseRecord($row);
        }

        return $userReminders;

    }
}
